import logging
import pandas as pd
import numpy as np
from scipy.stats import norm
import os
from Bio import Phylo,Entrez,AlignIO
from io import StringIO
import urllib.request
from urllib.error import HTTPError

Age_ranges = [(129.4,113.0),(121.4,113.0),(72.3,66.0),(121.4,113.0),(121.4,113.0),(113.0,105.68),(139.8,125.77),(56.0,33.9),(83.6,72.1),(124.85,124.35),(121.4,113.0),(28.4,23.03),(125.77,121.4),(93.9,89.5),(37.71,33.9),(113.0,100.5),(121.4,100.5),(113.0,100.5),(72.3,66.0),(113.0,100.5),(139.8,121.4),(145.0,100.5),(56.0,47.8),(110.0,99.0),(145.0,100.5),(83.8,71.9),(83.8,66.0),(145.0,100.5),(72.3,66.0),(113.0,100.5),(90.1,85.8),(66.0,56.0),(113.0,100.5),(83.8,71.9),(145.0,100.5),(66.0,23.03),(145.0,100.5),(113.0,100.5),(86.8,71.9),(113.0,93.9),(125.77,113.0),(90.1,85.8),(59.2,56.0),(66.0,63.3),(66.0,61.6),(66.0,61.6),(145.0,66.0),(56.0,47.8),(100.5,93.9),(33.9,28.4),(86.8,71.9),(145.0,100.5),(56.0,33.9),(145.0,100.5),(27.82,23.03),(83.8,71.9)]

Fern_Crown_Bounds = {'Eupolypods':100.5,'Aglaomorpha':7.246,'Athyriaceae':37.8,'Blechnaceae':37.8,'Cibotiaceae':33.9,'Culcitaceae':13.82,'Cyathea':5.333,'Drynaria':7.246,'Lygodiaceae':59.2,'Marattiaceae':174.1,'Pleopeltis':15.97,'Schizaeaceae':86.3,'Gleicheniaceae':100.5,'Pteridaceae':100.5,'Dennstaedtiaceae':72.1,'Thelypteridaceae':72.1,'Hymenophyllum':47.8,'Cyatheales':168.3,'Cyatheaceae':93.5,'Hymenophyllaceae':228,'Anemia':136.4,'Lygodium':66}

Fern_Stem_Bounds = {'Dryopteris':33.9,'Davallia':20.44,'Dicksonia':66,'Equisetum':145,'Equisetaceae':163.5,'Acrostichum':66,'Asplenium':47.8,'Cibotiaceae':86.3,'Cyatheaceae':129.4,'Cyatheales':228,'Cyclosorus':33.9,'Cystodiaceae':93.9,'Drynaria':7.246,'Elaphoglossum':20.44,'Equisetales':346.7,'Gleicheniaceae':272.95,'Goniophlebium':15.97,'Hymenophyllum':113,'Lophosoria':113,'Loxsomataceae':125,'Lygodium':167.7,'Marattiaceae':298.9,'Onoclea':56,'Ophioglossum':93.9,'Osmunda':93.9,'Osmundaceae':145,'Polypodium':27.82,'Regnellidium':83.6,'Schizaeaceae':163.5,'Dipteridaceae':168.3,'Thelypteridaceae':100.5,'Thyrsopteridaceae':145,'Trichomanoideae':93.9,'Athyriaceae':37.8,'Polystichum':23.03,'Matonia':100.5,'Athyrium':129.4,'Woodwardia':72.1,'Dryopteridaceae':129.4,'Blechnaceae':72.1,'Dennstaedtia':47.8,'Osmundales':299,'Marsileaceae':140.2,'Azolla':83.6,'Lindsaeaceae':100.5,'Athyrioids':37.2,'Polypodiaceae':33.9,'Matoniaceae':228,'Stromatopteris':89.3}

Fern_Stem_Bounds = {}

Orders = ['Nymphaeales','Austrobaileyales','Piperales','Canellales','Magnoliales','Laurales','Chloranthales','Commelinales','Zingiberales','Poales','Arecales','Asparagales','Liliales','Pandanales','Dioscoreales','Ceratophyllales','Ranunculales','Proteales','Trochodendrales','Buxales','Gunnerales','Fagales','Cucurbitales','Rosales','Fabales','Celastrales','Oxalidales','Malpighiales','Zygophyllales','Malvales','Brassicales','Huerteales','Sapindales','Crossosomatales','Myrtales','Geraniales','Vitales','Saxifragales','Dilleniales','Santalales','Caryophyllales','Cornales','Icacinales','Garryales','Boraginales','Gentianales','Lamiales','Solanales','Aquifoliales','Escalloniales','Asterales','Dipsacales','Paracryphiales','Apiales','Alismatales','Ericales']

Orders_Bounds_Previous = {'Nymphaeales':113.0,'Austrobaileyales':113.0,'Piperales':66.0,'Canellales':113.0,'Magnoliales':113.0,'Laurales':105.68,'Chloranthales':125.77,'Commelinales':33.9,'Zingiberales':72.1,'Poales':124.35,'Arecales':113.0,'Asparagales':23.03,'Liliales':121.4,'Pandanales':89.5,'Dioscoreales':33.9,'Ceratophyllales':100.5,'Ranunculales':100.5,'Proteales':100.5,'Trochodendrales':66.0,'Buxales':100.5,'Gunnerales':121.4,'Fagales':100.5,'Cucurbitales':47.8,'Rosales':99.0,'Fabales':100.5,'Celastrales':71.9,'Oxalidales':66.0,'Malpighiales':100.5,'Zygophyllales':66.0,'Malvales':100.5,'Brassicales':85.8,'Huerteales':56.0,'Sapindales':100.5,'Crossosomatales':71.9,'Myrtales':100.5,'Geraniales':23.03,'Vitales':100.5,'Saxifragales':100.5,'Dilleniales':71.9,'Santalales':93.9,'Caryophyllales':113.0,'Cornales':85.8,'Icacinales':56.0,'Garryales':63.3,'Boraginales':61.6,'Gentianales':61.6,'Lamiales':66.0,'Solanales':47.8,'Aquifoliales':93.9,'Escalloniales':28.4,'Asterales':71.9,'Dipsacales':100.5,'Paracryphiales':33.9,'Apiales':100.5,'Alismatales':23.03,'Ericales':71.9}

Orders_Bounds = {'Nymphaeales':113.0,'Austrobaileyales':113.0,'Piperales':72.1,'Canellales':125,'Magnoliales':113.0,'Laurales':113.0,'Chloranthales':123.0,'Commelinales':12.2,'Zingiberales':72.1,'Poales':66,'Arecales':83.6,'Asparagales':41.2,'Liliales':51,'Pandanales':56.0,'Dioscoreales':27.23,'Ceratophyllales':127.2,'Ranunculales':89.8,'Proteales':107.7,'Trochodendrales':66.0,'Buxales':110.8,'Gunnerales':0,'Fagales':93.9,'Cucurbitales':47.8,'Rosales':72.1,'Fabales':56.0,'Celastrales':46.3,'Oxalidales':80.7,'Malpighiales':86.3,'Zygophyllales':66.0,'Malvales':72.1,'Brassicales':86.3,'Huerteales':41.2,'Sapindales':89.8,'Crossosomatales':56.0,'Myrtales':86.3,'Geraniales':15.97,'Vitales':61.6,'Saxifragales':83.6,'Dilleniales':47.8,'Santalales':80.7,'Caryophyllales':72.1,'Cornales':86.3,'Icacinales':47.8,'Garryales':47.8,'Boraginales':47.8,'Gentianales':48.5,'Lamiales':41.2,'Solanales':52.22,'Aquifoliales':61.6,'Escalloniales':28.4,'Asterales':72.1,'Dipsacales':34.07,'Paracryphiales':33.9,'Apiales':66.0,'Alismatales':66,'Ericales':86.3}

Family_Bounds = {'Juglandaceae':83.4,'Myricaceae':83.4,'Betulaceae':83.4,'Casuarinaceae':56.0,'Nothofagaceae':83.4,'Schisandraceae':100.5,'Nymphaeaceae':100.5,'Ceratophyllaceae':121.4,'Chloranthaceae':113.0,'Winteraceae':121.4,'Lauraceae':100.5,'Magnoliaceae':93.9,'Myristicaceae':61.6,'Aristolochiaceae':71.9,'Saururaceae':44.3,'Alismataceae':71.9,'Aponogetonaceae':81.13,'Araceae':113.0,'Posidoniaceae':66.0}

Family_Upper_Bounds_FollowYang = {'Juglandaceae':128.63,'Myricaceae':128.63,'Betulaceae':128.63,'Casuarinaceae':128.63,'Nothofagaceae':83.4,'Schisandraceae':247.2,'Nymphaeaceae':247.2,'Ceratophyllaceae':128.63,'Chloranthaceae':247.2,'Winteraceae':247.2,'Lauraceae':247.2,'Magnoliaceae':247.2,'Myristicaceae':247.2,'Aristolochiaceae':247.2,'Saururaceae':247.2,'Alismataceae':128.63,'Aponogetonaceae':128.63,'Araceae':128.63,'Posidoniaceae':128.63}

Orders_Upper_Stratum = {'Nymphaeales':'Hauterivian','Austrobaileyales':'Valanginian','Piperales':'Campanian','Canellales':'Santonian','Magnoliales':'Hauterivian','Laurales':'Tithonian','Chloranthales':'Tithonian','Commelinales':'Thanetian','Zingiberales':'Santonian','Poales':'Tithonian','Arecales':'Ladinian','Asparagales':'Maastrichtian','Liliales':'Hauterivian','Pandanales':'Cenomanian','Dioscoreales':'Bartonian','Ceratophyllales':'Aptian','Ranunculales':'Barremian','Proteales':'Tithonian','Trochodendrales':'Campanian','Buxales':'Aptian','Gunnerales':'Tithonian','Fagales':'Tithonian','Cucurbitales':'Maastrichtian','Rosales':'Tithonian','Fabales':'Tithonian','Celastrales':'Albian','Oxalidales':'Albian','Malpighiales':'Tithonian','Zygophyllales':'Campanian','Malvales':'Tithonian','Brassicales':'Maastrichtian','Huerteales':'Maastrichtian','Sapindales':'Aptian','Crossosomatales':'Aptian','Myrtales':'Tithonian','Geraniales':'Maastrichtian','Vitales':'Tithonian','Saxifragales':'Aptian','Dilleniales':'Albian','Santalales':'Aptian','Caryophyllales':'Hauterivian','Cornales':'Albian','Icacinales':'Valanginian','Garryales':'Maastrichtian','Boraginales':'Maastrichtian','Gentianales':'Maastrichtian','Lamiales':'Tithonian','Solanales':'Maastrichtian','Aquifoliales':'Tithonian','Escalloniales':'Thanetian','Asterales':'Turonian','Dipsacales':'Tithonian','Paracryphiales':'Thanetian','Apiales':'Tithonian','Angiospermae':'Olenekian','Alismatales':'Rupelian','Ericales':'Santonian'}
Orders_Upper_Bounds = {'Nymphaeales':132.6,'Austrobaileyales':139.8,'Piperales':83.8,'Canellales':86.8,'Magnoliales':132.6,'Laurales':149.9,'Chloranthales':149.9,'Commelinales':59.2,'Zingiberales':86.8,'Poales':149.9,'Arecales':242,'Asparagales':72.3,'Liliales':132.6,'Pandanales':100.5,'Dioscoreales':41.2,'Ceratophyllales':121.4,'Ranunculales':125.77,'Proteales':149.9,'Trochodendrales':83.8,'Buxales':121.4,'Gunnerales':149.9,'Fagales':149.9,'Cucurbitales':72.3,'Rosales':149.9,'Fabales':149.9,'Celastrales':113.0,'Oxalidales':113.0,'Malpighiales':149.9,'Zygophyllales':83.8,'Malvales':149.9,'Brassicales':72.3,'Huerteales':72.3,'Sapindales':121.4,'Crossosomatales':121.4,'Myrtales':149.9,'Geraniales':72.3,'Vitales':149.9,'Saxifragales':121.4,'Dilleniales':113.0,'Santalales':121.4,'Caryophyllales':132.6,'Cornales':113.0,'Icacinales':139.8,'Garryales':72.3,'Boraginales':72.3,'Gentianales':72.3,'Lamiales':149.9,'Solanales':72.3,'Aquifoliales':149.9,'Escalloniales':59.2,'Asterales':93.9,'Dipsacales':149.9,'Paracryphiales':59.2,'Apiales':149.9,'Alismatales':33.9,'Ericales':86.8}

Major_Clades_Upper_Stratum = {'Mesangiospermae':'Olenekian','Magnoliidae':'Hettangian','Eudicotyledoneae':'Hettangian','Monocotyledoneae':'Hettangian','Ceratophyllales':'Hettangian','Austrobaileyales':'Olenekian','Nymphaeales':'Olenekian','Amborellales':'Olenekian'}

Major_Clades_Upper_Bounds = {'Mesangiospermae':247.2,'Magnoliidae':201.6,'Eudicotyledoneae':201.6,'Monocotyledoneae':201.6,'Ceratophyllales':201.6,'Austrobaileyales':247.2,'Nymphaeales':247.2,'Amborellales':247.2}

Clades_Upper_Bounds = {'Nymphaeales':247.2,'Austrobaileyales':247.2,'Piperales':201.6,'Canellales':201.6,'Magnoliales':201.6,'Laurales':201.6,'Chloranthales':201.6,'Commelinales':201.6,'Zingiberales':201.6,'Poales':201.6,'Arecales':201.6,'Asparagales':201.6,'Liliales':201.6,'Pandanales':201.6,'Dioscoreales':201.6,'Ceratophyllales':201.6,'Ranunculales':201.6,'Proteales':201.6,'Trochodendrales':201.6,'Buxales':201.6,'Gunnerales':201.6,'Fagales':201.6,'Cucurbitales':201.6,'Rosales':201.6,'Fabales':201.6,'Celastrales':201.6,'Oxalidales':201.6,'Malpighiales':201.6,'Zygophyllales':201.6,'Malvales':201.6,'Brassicales':201.6,'Huerteales':201.6,'Sapindales':201.6,'Crossosomatales':201.6,'Myrtales':201.6,'Geraniales':201.6,'Vitales':201.6,'Saxifragales':201.6,'Dilleniales':201.6,'Santalales':201.6,'Caryophyllales':201.6,'Cornales':201.6,'Icacinales':201.6,'Garryales':201.6,'Boraginales':201.6,'Gentianales':201.6,'Lamiales':201.6,'Solanales':201.6,'Aquifoliales':201.6,'Escalloniales':201.6,'Asterales':201.6,'Dipsacales':201.6,'Paracryphiales':201.6,'Apiales':201.6,'Alismatales':201.6,'Ericales':201.6}

Clades_Upper_Bounds_FollowYang = {'Nymphaeales':247.2,'Austrobaileyales':247.2,'Piperales':247.2,'Canellales':247.2,'Magnoliales':247.2,'Laurales':247.2,'Chloranthales':247.2,'Commelinales':128.63,'Zingiberales':128.63,'Poales':128.63,'Arecales':128.63,'Asparagales':128.63,'Liliales':128.63,'Pandanales':128.63,'Dioscoreales':128.63,'Ceratophyllales':128.63,'Ranunculales':128.63,'Proteales':128.63,'Trochodendrales':128.63,'Buxales':128.63,'Gunnerales':128.63,'Fagales':128.63,'Cucurbitales':128.63,'Rosales':128.63,'Fabales':128.63,'Celastrales':128.63,'Oxalidales':128.63,'Malpighiales':128.63,'Zygophyllales':128.63,'Malvales':128.63,'Brassicales':128.63,'Huerteales':128.63,'Sapindales':128.63,'Crossosomatales':128.63,'Myrtales':128.63,'Geraniales':128.63,'Vitales':128.63,'Saxifragales':128.63,'Dilleniales':128.63,'Santalales':128.63,'Caryophyllales':128.63,'Cornales':128.63,'Icacinales':128.63,'Garryales':128.63,'Boraginales':128.63,'Gentianales':128.63,'Lamiales':128.63,'Solanales':128.63,'Aquifoliales':128.63,'Escalloniales':128.63,'Asterales':128.63,'Dipsacales':128.63,'Paracryphiales':128.63,'Apiales':128.63,'Alismatales':128.63,'Ericales':128.63,'Amborellales':247.2}

APG_IV_Modified_Tree = "(Amborellales,(Nymphaeales,(Austrobaileyales,((Acorales,(Alismatales,(Petrosaviales,((Dioscoreales,Pandanales),(Liliales,(Asparagales,(Arecales,(Poales,(Zingiberales,Commelinales))))))))),((Chloranthales,((Laurales,Magnoliales),(Canellales,Piperales))),(Ceratophyllales,(Ranunculales,(Proteales,(Trochodendrales,(Buxales,(Gunnerales,(Dilleniales,(Saxifragales,(Vitales,(((Myrtales,Geraniales),(Crossosomatales,(Picramniales,(Sapindales,(Huerteales,(Malvales,Brassicales)))))),(Zygophyllales,((Celastrales,(Malpighiales,Oxalidales)),(Fabales,(Rosales,(Fagales,Cucurbitales)))))))),(Berberidopsidales,(Santalales,(Caryophyllales,(Cornales,(Ericales,((Aquifoliales,(Asterales,Escalloniales,(Bruniales,(Apiales,(Dipsacales,Paracryphiales))))),(Icacinales,(Metteniusales,(Garryales,(Boraginales,Gentianales,Vahliales,Lamiales,Solanales))))))))))))))))))))));"

Updated_Tree_in_Paper = "(Amborellales,(Nymphaeales,(Austrobaileyales,((Acorales,(Alismatales,(Petrosaviales,((Pandanales,Dioscoreales),(Liliales,(Asparagales,((Arecales,(Zingiberales,Commelinales)),Poales))))))),(((Ranunculales,(Proteales,((Trochodendrales,Buxales),(Gunnerales,((Caryophyllales,((Cornales,Ericales),(((Apiales,Dipsacales),Asterales),(Aquifoliales,((Solanales,(Gentianales,Boraginales)),Lamiales))))),(Santalales,(Vitales,(Saxifragales,(Crossosomatales,(Myrtales,(((Celastrales,Malpighiales),(Oxalidales,(Sapindales,(Malvales,Brassicales)))),((Cucurbitales,Rosales),(Fagales,Fabales))))))))))))),Ceratophyllales),(Chloranthales,((Piperales,Canellales),(Magnoliales,Laurales))))))));"


APG_IV_Modified_Family_Tree = APG_IV_Modified_Tree.replace("Fagales","(Nothofagaceae,(Fagaceae,((Myricaceae,(Juglandaceae,Rhoipteleaceae)),(Casuarinaceae,(Betulaceae,Ticodendraceae)))))").replace("Austrobaileyales","(Austrobaileyaceae,(Trimeniaceae,Schisandraceae))").replace("Nymphaeales","(Hydatellaceae,(Cabombaceae,Nymphaeaceae))").replace("Ceratophyllales","Ceratophyllaceae").replace("Chloranthales","Chloranthaceae").replace("Canellales","(Canellaceae,Winteraceae)").replace("Laurales","(Calycanthaceae,((Siparunaceae,(Gomortegaceae,Atherospermataceae)),(Monimiaceae,(Hernandiaceae,Lauraceae))))").replace("Magnoliales","(Myristicaceae,(Magnoliaceae,((Degeneriaceae,Himantandraceae),(Eupomatiaceae,Annonaceae))))").replace("Piperales","(Aristolochiaceae,(Piperaceae,Saururaceae))").replace("Alismatales","(Araceae,(Tofieldiaceae,(((Hydrocharitaceae,Butomaceae),Alismataceae),(Scheuchzeriaceae,(Aponogetonaceae,(Juncaginaceae,(Maundiaceae,((Zosteraceae,Potamogetonaceae),(Posidoniaceae,(Ruppiaceae,Cymodoceaceae))))))))))").replace("Asparagales","(Orchidaceae,((Boryaceae,(Blandfordiaceae,(Lanariaceae,(Asteliaceae,Hypoxidaceae)))),((Ixioliriaceae,Tecophilaeaceae),(Doryanthaceae,(Iridaceae,(Xeronemataceae,(Xanthorrhoeaceae,(Amaryllidaceae,Asparagaceae))))))))")

Codon_Table = {'TTT':'F','TTC':'F','TTA':'L','TTG':'L','CTT':'L','CTC':'L','CTA':'L','CTG':'L','ATT':'I','ATC':'I','ATA':'I','ATG':'M','GTT':'V','GTC':'V','GTA':'V','GTG':'V','TCT':'S','TCC':'S','TCA':'S','TCG':'S','CCT':'P','CCC':'P','CCA':'P','CCG':'P','ACT':'T','ACC':'T','ACA':'T','ACG':'T','GCT':'A','GCC':'A','GCA':'A','GCG':'A','TAT':'Y','TAC':'Y','TAA':'','TAG':'','CAT':'H','CAC':'H','CAA':'Q','CAG':'Q','AAT':'N','AAC':'N','AAA':'K','AAG':'K','GAT':'D','GAC':'D','GAA':'E','GAG':'E','TGT':'C','TGC':'C','TGA':'','TGG':'W','CGT':'R','CGC':'R','CGA':'R','CGG':'R','AGT':'S','AGC':'S','AGA':'R','AGG':'R','GGT':'G','GGC':'G','GGA':'G','GGG':'G'}

Family_Affinity = {'Abrus precatorius': 'Fabaceae', 'Acacia pycnantha': 'Fabaceae', 'Acanthochlamys bracteata': 'Velloziaceae', 'Acer pseudosieboldianum': 'Sapindaceae', 'Acer yangbiense': 'Sapindaceae', 'Achnatherum splendens': 'Poaceae', 'Acorus americanus': 'Acoraceae', 'Acorus tatarinowii': 'Acoraceae', 'Actinidia chinensis': 'Actinidiaceae', 'Actinidia eriantha': 'Actinidiaceae', 'Aegiceras corniculatum': 'Primulaceae', 'Aegilops tauschii': 'Poaceae', 'Aeschynomene evenia': 'Fabaceae', 'Aethionema arabicum': 'Brassicaceae', 'Akebia trifoliata': 'Lardizabalaceae', 'Allium sativum': 'Amaryllidaceae', 'Alternaria oxytropis': 'Pleosporaceae', 'Amaranthus cruentus': 'Amaranthaceae', 'Amaranthus hybridus': 'Amaranthaceae', 'Amaranthus hypochondriacus': 'Amaranthaceae', 'Amaranthus palmeri': 'Amaranthaceae', 'Amaranthus tuberculatus': 'Amaranthaceae', 'Amborella trichopoda': 'Amborellaceae', 'Ammopiptanthus nanus': 'Fabaceae', 'Amorphophallus konjac': 'Araceae', 'Amphicarpaea edgeworthii': 'Fabaceae', 'Ananas comosus': 'Bromeliaceae', 'Andrographis paniculata': 'Acanthaceae', 'Angelica sinensis': 'Apiaceae', 'Antirrhinum majus': 'Plantaginaceae', 'Apium graveolens': 'Apiaceae', 'Apostasia shenzhenica': 'Orchidaceae', 'Aquilaria sinensis': 'Thymelaeaceae', 'Aquilegia coerulea': 'Ranunculaceae', 'Aquilegia oxysepala': 'Ranunculaceae', 'Arabidopsis lyrata': 'Brassicaceae', 'Arabidopsis thaliana': 'Brassicaceae', 'Arabis alpina': 'Brassicaceae', 'Arachis duranensis': 'Fabaceae', 'Arachis hypogaea': 'Fabaceae', 'Aralia elata': 'Araliaceae', 'Arctium lappa': 'Asteraceae', 'Argentina anserina': 'Rosaceae', 'Aristolochia contorta': 'Aristolochiaceae', 'Aristolochia fimbriata': 'Aristolochiaceae', 'Artemisia annua': 'Asteraceae', 'Artocarpus nanchuanensis': 'Moraceae', 'Asparagus officinalis': 'Asparagaceae', 'Asparagus setaceus': 'Asparagaceae', 'Atriplex hortensis': 'Chenopodiaceae', 'Avena insularis': 'Poaceae', 'Avena sativa': 'Poaceae', 'Averrhoa carambola': 'Oxalidaceae', 'Avicennia marina': 'Acanthaceae', 'Bauhinia variegata': 'Fabaceae', 'Begonia darthvaderiana': 'Begoniaceae', 'Begonia fuchsioides': 'Begoniaceae', 'Benincasa hispida': 'Cucurbitaceae', 'Beta patula': 'Chenopodiaceae', 'Beta vulgaris': 'Chenopodiaceae', 'Betula pendula': 'Betulaceae', 'Boechera stricta': 'Brassicaceae', 'Boehmeria nivea': 'Urticaceae', 'Bombax ceiba': 'Malvaceae', 'Bonia amplexicaulis': 'Poaceae', 'Brachypodium distachyon': 'Poaceae', 'Brachypodium hybridum': 'Poaceae', 'Brassica napus': 'Brassicaceae', 'Brassica oleracea': 'Brassicaceae', 'Brassica rapa': 'Brassicaceae', 'Bretschneidera sinensis': 'Akaniaceae', 'Bruguiera sexangula': 'Rhizophoraceae', 'Buddleja alternifolia': 'Scrophulariaceae', 'Buxus austroyunnanensis': 'Buxaceae', 'Buxus sinica': 'Buxaceae', 'Cajanus cajan': 'Fabaceae', 'Calamus simplicifolius': 'Arecaceae', 'Callicarpa americana': 'Lamiaceae', 'Calotropis gigantea': 'Apocynaceae', 'Camelina sativa': 'Brassicaceae', 'Camellia chekiangoleosa': 'Theaceae', 'Camellia oleifera': 'Theaceae', 'Camptotheca acuminata': 'Nyssaceae', 'Cannabis sativa': 'Cannabaceae', 'Capparis spinosa': 'Capparaceae', 'Capsella rubella': 'Brassicaceae', 'Capsicum annuum': 'Solanaceae', 'Capsicum baccatum': 'Solanaceae', 'Capsicum chinense': 'Solanaceae', 'Cardamine hirsuta': 'Brassicaceae', 'Carex cristatella': 'Cyperaceae', 'Carex scoparia': 'Cyperaceae', 'Carica papaya': 'Caricaceae', 'Carpinus fangiana': 'Betulaceae', 'Carthamus tinctorius': 'Asteraceae', 'Carya illinoinensis': 'Juglandaceae', 'Castanea mollissima': 'Fagaceae', 'Casuarina glauca': 'Casuarinaceae', 'Catharanthus roseus': 'Apocynaceae', 'Cephalotus follicularis': 'Cephalotaceae', 'Ceratophyllum demersum': 'Ceratophyllaceae', 'Cercis chinensis': 'Fabaceae', 'Chenopodium quinoa': 'Chenopodiaceae', 'Chimonanthus salicifolius': 'Calycanthaceae', 'Chiococca alba': 'Rubiaceae', 'Chloranthus sessilifolius': 'Chloranthaceae', 'Chloranthus spicatus': 'Chloranthaceae', 'Chrysanthemum seticuspe': 'Asteraceae', 'Cicer arietinum': 'Fabaceae', 'Cichorium endivia': 'Asteraceae', 'Cichorium intybus': 'Asteraceae', 'Cinchona pubescens': 'Rubiaceae', 'Cinnamomum kanehirae': 'Lauraceae', 'Citrullus lanatus': 'Cucurbitaceae', 'Citrus clementina': 'Rutaceae', 'Citrus maxima': 'Rutaceae', 'Citrus sinensis': 'Rutaceae', 'Cleome violacea': 'Cleomaceae', 'Clethra arborea': 'Clethraceae', 'Cocos nucifera': 'Arecaceae', 'Coffea canephora': 'Rubiaceae', 'Coffea humblotiana': 'Rubiaceae', 'Coptis chinensis': 'Ranunculaceae', 'Corchorus capsularis': 'Malvaceae', 'Coriandrum sativum': 'Apiaceae', 'Corydalis tomentella': 'Papaveraceae', 'Corylus heterophylla': 'Betulaceae', 'Crataegus pinnatifida': 'Rosaceae', 'Cremastra appendiculata': 'Orchidaceae', 'Cucumis melo': 'Cucurbitaceae', 'Cucumis sativus': 'Cucurbitaceae', 'Cucurbita argyrosperma': 'Cucurbitaceae', 'Cucurbita maxima': 'Cucurbitaceae', 'Cuscuta campestris': 'Convolvulaceae', 'Cycas panzhihuaensis': 'Cycadaceae', 'Cyclocarya paliurus': 'Juglandaceae', 'Cymodocea nodosa': 'Cymodoceaceae', 'Cynara cardunculus': 'Asteraceae', 'Daemonorops jenkinsiana': 'Arecaceae', 'Dalbergia odorifera': 'Fabaceae', 'Datisca glomerata': 'Datiscaceae', 'Datura stramonium': 'Solanaceae', 'Daucus carota': 'Apiaceae', 'Davidia involucrata': 'Nyssaceae', 'Dendrobium nobile': 'Orchidaceae', 'Dianthus caryophyllus': 'Caryophyllaceae', 'Dichanthelium oligosanthes': 'Poaceae', 'Dimocarpus longan': 'Sapindaceae', 'Dioscorea alata': 'Dioscoreaceae', 'Dioscorea rotundata': 'Dioscoreaceae', 'Diospyros lotus': 'Ebenaceae', 'Draba nivalis': 'Brassicaceae', 'Dryas drummondii': 'Rosaceae', 'Durio zibethinus': 'Malvaceae', 'Echinochloa crus-galli': 'Poaceae', 'Elaeagnus angustifolia': 'Elaeagnaceae', 'Elaeis guineensis': 'Arecaceae', 'Ensete glaucum': 'Musaceae', 'Ensete ventricosum': 'Musaceae', 'Entada phaseoloides': 'Fabaceae', 'Epimedium pubescens': 'Berberidaceae', 'Eragrostis nindensis': 'Poaceae', 'Eragrostis tef': 'Poaceae', 'Erigeron canadensis': 'Asteraceae', 'Eriobotrya japonica': 'Rosaceae', 'Eruca sativa': 'Brassicaceae', 'Erysimum cheiranthoides': 'Brassicaceae', 'Eucalyptus grandis': 'Myrtaceae', 'Euphorbia lathyris': 'Euphorbiaceae', 'Eupristina verticillata': 'Agaonidae', 'Euryale ferox': 'Nymphaeaceae', 'Euscaphis japonica': 'Staphyleaceae', 'Eutrema salsugineum': 'Brassicaceae', 'Fagopyrum tataricum': 'Polygonaceae', 'Fagus sylvatica': 'Fagaceae', 'Faidherbia albida': 'Fabaceae', 'Ficus hispida': 'Moraceae', 'Ficus microcarpa': 'Moraceae', 'Fragaria pentaphylla': 'Rosaceae', 'Fragaria vesca': 'Rosaceae', 'Fraxinus pennsylvanica': 'Oleaceae', 'Gardenia jasminoides': 'Rubiaceae', 'Genlisea aurea': 'Lentibulariaceae', 'Gilia yorkii': 'Polemoniaceae', 'Glycine max': 'Fabaceae', 'Glycine soja': 'Fabaceae', 'Gossypium barbadense': 'Malvaceae', 'Gossypium hirsutum': 'Malvaceae', 'Gossypium raimondii': 'Malvaceae', 'Guadua angustifolia': 'Poaceae', 'Gypsophila paniculata': 'Caryophyllaceae', 'Haloxylon ammodendron': 'Chenopodiaceae', 'Handroanthus impetiginosus': 'Bignoniaceae', 'Helianthus annuus': 'Asteraceae', 'Hevea brasiliensis': 'Euphorbiaceae', 'Hibiscus cannabinus': 'Malvaceae', 'Hippophae rhamnoides': 'Elaeagnaceae', 'Hordeum vulgare': 'Poaceae', 'Humulus lupulus': 'Cannabaceae', 'Hydrangea macrophylla': 'Hydrangeaceae', 'Hylocereus undatus': 'Cactaceae', 'Ilex polyneura': 'Aquifoliaceae', 'Ipomoea nil': 'Convolvulaceae', 'Jacaranda mimosifolia': 'Bignoniaceae', 'Jasminum sambac': 'Oleaceae', 'Jatropha curcas': 'Euphorbiaceae', 'Juglans mandshurica': 'Juglandaceae', 'Juglans regia': 'Juglandaceae', 'Juglans sigillata': 'Juglandaceae', 'Juncus effusus': 'Juncaceae', 'Juncus inflexus': 'Juncaceae', 'Kalanchoe fedtschenkoi': 'Crassulaceae', 'Kandelia candel': 'Rhizophoraceae', 'Kandelia obovata': 'Rhizophoraceae', 'Kingdonia uniflora': 'Circaeasteraceae', 'Lablab purpureus': 'Fabaceae', 'Lactuca sativa': 'Asteraceae', 'Leersia perrieri': 'Poaceae', 'Lemna minor': 'Araceae', 'Lemna minuta': 'Araceae', 'Lens culinaris': 'Fabaceae', 'Lens ervoides': 'Fabaceae', 'Lepidium sativum': 'Brassicaceae', 'Lindenbergia luchunensis': 'Orobanchaceae', 'Linum usitatissimum': 'Linaceae', 'Liriodendron chinense': 'Magnoliaceae', 'Liriodendron tulipifera': 'Magnoliaceae', 'Litchi chinensis': 'Sapindaceae', 'Lithospermum erythrorhizon': 'Boraginaceae', 'Lobularia maritima': 'Brassicaceae', 'Iochroma cyaneum': 'Solanaceae', 'Lonicera japonica': 'Caprifoliaceae', 'Lotus japonicus': 'Fabaceae', 'Lupinus angustifolius': 'Fabaceae', 'Macadamia integrifolia': 'Proteaceae', 'Macadamia jansenii': 'Proteaceae', 'Macleaya cordata': 'Papaveraceae', 'Macrotyloma uniflorum': 'Fabaceae', 'Magnolia biondii': 'Magnoliaceae', 'Malania oleifera': 'Ximeniaceae', 'Malus domestica': 'Rosaceae', 'Malus sieversii': 'Rosaceae', 'Mangifera indica': 'Anacardiaceae', 'Manihot esculenta': 'Euphorbiaceae', 'Medicago sativa': 'Fabaceae', 'Medicago truncatula': 'Fabaceae', 'Megacarpaea delavayi': 'Brassicaceae', 'Megadenia pygmaea': 'Brassicaceae', 'Melaleuca alternifolia': 'Myrtaceae', 'Melastoma dodecandrum': 'Melastomataceae', 'Melilotus albus': 'Fabaceae', 'Microthlaspi erraticum': 'Brassicaceae', 'Mikania micrantha': 'Asteraceae', 'Mimulus guttatus': 'Phrymaceae', 'Miscanthus lutarioriparius': 'Poaceae', 'Morella rubra': 'Myricaceae', 'Moringa oleifera': 'Moringaceae', 'Morus notabilis': 'Moraceae', 'Mucuna pruriens': 'Fabaceae', 'Musa acuminata': 'Musaceae', 'Musa balbisiana': 'Musaceae', 'Nelumbo nucifera': 'Nelumbonaceae', 'Nephelium lappaceum': 'Sapindaceae', 'Nicotiana attenuata': 'Solanaceae', 'Nicotiana tabacum': 'Solanaceae', 'Nicotiana tomentosiformis': 'Solanaceae', 'Nymphaea colorata': 'Nymphaeaceae', 'Nymphaea thermarum': 'Nymphaeaceae', 'Nymphoides indica': 'Menyanthaceae', 'Nyssa sinensis': 'Nyssaceae', 'Nyssa yunnanensis': 'Nyssaceae', 'Ocimum basilicum': 'Lamiaceae', 'Olea europaea': 'Oleaceae', 'Olyra latifolia': 'Poaceae', 'Ophiorrhiza pumila': 'Rubiaceae', 'Origanum majorana': 'Lamiaceae', 'Origanum vulgare': 'Lamiaceae', 'Orobanche cumana': 'Orobanchaceae', 'Oropetium thomaeum': 'Poaceae', 'Orychophragmus violaceus': 'Brassicaceae', 'Oryza glaberrima': 'Poaceae', 'Oryza punctata': 'Poaceae', 'Oryza rufipogon': 'Poaceae', 'Oryza sativa': 'Poaceae', 'Oxytropis ochrocephala': 'Fabaceae', 'Paeonia ostii': 'Paeoniaceae', 'Paeonia suffruticosa': 'Paeoniaceae', 'Panax ginseng': 'Araliaceae', 'Panax japonicus': 'Araliaceae', 'Panax quinquefolius': 'Araliaceae', 'Panax stipuleanatus': 'Araliaceae', 'Panicum virgatum': 'Poaceae', 'Papaver rhoeas': 'Papaveraceae', 'Papaver setigerum': 'Papaveraceae', 'Papaver somniferum': 'Papaveraceae', 'Parasponia andersonii': 'Cannabaceae', 'Passiflora edulis': 'Passifloraceae', 'Passiflora organensis': 'Passifloraceae', 'Paulownia fortunei': 'Paulowniaceae', 'Pennisetum glaucum': 'Poaceae', 'Perilla frutescens': 'Lamiaceae', 'Persea americana': 'Lauraceae', 'Petunia axillaris': 'Solanaceae', 'Phalaenopsis equestris': 'Orchidaceae', 'Pharus latifolius': 'Poaceae', 'Phaseolus vulgaris': 'Fabaceae', 'Phelipanche aegyptiaca': 'Orobanchaceae', 'Phoenix dactylifera': 'Arecaceae', 'Phragmites australis': 'Poaceae', 'Phtheirospermum japonicum': 'Orobanchaceae', 'Physalis grisea': 'Solanaceae', 'Physalis pruinosa': 'Solanaceae', 'Picrorhiza kurrooa': 'Plantaginaceae', 'Piper nigrum': 'Piperaceae', 'Pistacia vera': 'Anacardiaceae', 'Pistia stratiotes': 'Araceae', 'Pisum sativm': 'Fabaceae', 'Platycarya strobilacea': 'Juglandaceae', 'Pogostemon cablin': 'Lamiaceae', 'Poncirus trifoliata': 'Rutaceae', 'Populus trichocarpa': 'Salicaceae', 'Populus wilsonii': 'Salicaceae', 'Portulaca amilis': 'Portulacaceae', 'Posidonia oceanica': 'Posidoniaceae', 'Potamogeton acutifolius': 'Potamogetonaceae', 'Primula veris': 'Primulaceae', 'Protea cynaroides': 'Proteaceae', 'Prunus mume': 'Rosaceae', 'Prunus persica': 'Rosaceae', 'Punica granatum': 'Lythraceae', 'Pyrus betulifolia': 'Rosaceae', 'Pyrus bretschneideri': 'Rosaceae', 'Quercus acutissima': 'Fagaceae', 'Quercus robur': 'Fagaceae', 'Quercus rubra': 'Fagaceae', 'Raddia guianensis': 'Poaceae', 'Raphanus sativus': 'Brassicaceae', 'Rhizophora apiculata': 'Rhizophoraceae', 'Rhodiola crenulata': 'Crassulaceae', 'Rhododendron griersonianum': 'Ericaceae', 'Rhododendron simsii': 'Ericaceae', 'Rhynchospora breviuscula': 'Cyperaceae', 'Rhynchospora pubera': 'Cyperaceae', 'Rhynchospora tenuis': 'Cyperaceae', 'Ricinus communis': 'Euphorbiaceae', 'Roridula gorgonias': 'Roridulaceae', 'Rosa chinensis': 'Rosaceae', 'Rosa rugosa': 'Rosaceae', 'Rosmarinus officinalis': 'Lamiaceae', 'Rubus corchorifolius': 'Rosaceae', 'Rubus occidentalis': 'Rosaceae', 'Saccharum officinarum': 'Poaceae', 'Salix arbutifolia': 'Salicaceae', 'Salix chaenomeloides': 'Salicaceae', 'Salvia splendens': 'Lamiaceae', 'Sapindus mukorossi': 'Sapindaceae', 'Schrenkiella parvula': 'Brassicaceae', 'Sclerocarya birrea': 'Anacardiaceae', 'Scutellaria baicalensis': 'Lamiaceae', 'Secale cereale': 'Poaceae', 'Sedum album': 'Crassulaceae', 'Senna tora': 'Fabaceae', 'Sesamum indicum': 'Pedaliaceae', 'Setaria italica': 'Poaceae', 'Setaria viridis': 'Poaceae', 'Shorea leprosula': 'Dipterocarpaceae', 'Simmondsia chinensis': 'Simmondsiaceae', 'Sinapis alba': 'Brassicaceae', 'Siraitia grosvenorii': 'Cucurbitaceae', 'Smallanthus sonchifolius': 'Asteraceae', 'Solanum lycopersicum': 'Solanaceae', 'Solanum pennellii': 'Solanaceae', 'Solanum tuberosum': 'Solanaceae', 'Sonneratia alba': 'Lythraceae', 'Sonneratia caseolaris': 'Lythraceae', 'Sorbus pohuashanensis': 'Rosaceae', 'Sorghum bicolor': 'Poaceae', 'Spatholobus suberectus': 'Fabaceae', 'Spinacia oleracea': 'Chenopodiaceae', 'Spirodela intermedia': 'Araceae', 'Spirodela polyrhiza': 'Araceae', 'Stellera chamaejasme': 'Thymelaeaceae', 'Stevia rebaudiana': 'Asteraceae', 'Streptocarpus rexii': 'Gesneriaceae', 'Streptochaeta angustifolia': 'Poaceae', 'Striga asiatica': 'Orobanchaceae', 'Striga hermonthica': 'Orobanchaceae', 'Strobilanthes cusia': 'Acanthaceae', 'Suaeda aralocaspica': 'Chenopodiaceae', 'Syringa oblata': 'Oleaceae', 'Syzygium aromaticum': 'Myrtaceae', 'Taraxacum kok-saghyz': 'Asteraceae', 'Taraxacum mongolicum': 'Asteraceae', 'Tarenaya hassleriana': 'Cleomaceae', 'Tectona grandis': 'Lamiaceae', 'Telopea speciosissima': 'Proteaceae', 'Tetracentron sinense': 'Trochodendraceae', 'Thalassia testudinum': 'Hydrocharitaceae', 'Thalictrum thalictroides': 'Ranunculaceae', 'Thellungiella parvula': 'Brassicaceae', 'Theobroma cacao': 'Malvaceae', 'Thinopyrum intermedium': 'Poaceae', 'Thlaspi arvense': 'Brassicaceae', 'Thymus quinquecostatus': 'Lamiaceae', 'Toona sinensis': 'Meliaceae', 'Trema orientale': 'Cannabaceae', 'Trichopus zeylanicus': 'Dioscoreaceae', 'Trifolium pratense': 'Fabaceae', 'Trifolium subterraneum': 'Fabaceae', 'Tripterygium wilfordii': 'Celastraceae', 'Triticum aestivum': 'Poaceae', 'Triticum turgidum': 'Poaceae', 'Trochodendron aralioides': 'Trochodendraceae', 'Utricularia gibba': 'Lentibulariaceae', 'Vaccinium caesariense': 'Ericaceae', 'Vaccinium corymbosum': 'Ericaceae', 'Vernicia fordii': 'Euphorbiaceae', 'Vicia sativa': 'Fabaceae', 'Vigna angularis': 'Fabaceae', 'Vigna radiata': 'Fabaceae', 'Vigna subterranea': 'Fabaceae', 'Vitellaria paradoxa': 'Sapotaceae', 'Vitis arizonica': 'Vitaceae', 'Vitis vinifera': 'Vitaceae', 'Wolffia australiana': 'Araceae', 'Wurfbainia villosa': 'Zingiberaceae', 'Xanthoceras sorbifolium': 'Sapindaceae', 'Zanthoxylum armatum': 'Rutaceae', 'Zanthoxylum bungeanum': 'Rutaceae', 'Zea mays': 'Poaceae', 'Zingiber officinale': 'Zingiberaceae', 'Zizania latifolia': 'Poaceae', 'Zizania palustris': 'Poaceae', 'Ziziphus jujuba var.spinosa': 'Rhamnaceae', 'Zostera marina': 'Zosteraceae', 'Zostera muelleri': 'Zosteraceae', 'Zoysia japonica': 'Poaceae', 'Zoysia matrella': 'Poaceae'}

Order_Affinity = {'Abrus precatorius': 'Fabales', 'Acacia pycnantha': 'Fabales', 'Acanthochlamys bracteata': 'Pandanales', 'Acer pseudosieboldianum': 'Sapindales', 'Acer yangbiense': 'Sapindales', 'Achnatherum splendens': 'Poales', 'Acorus americanus': 'Acorales', 'Acorus tatarinowii': 'Acorales', 'Actinidia chinensis': 'Ericales', 'Actinidia eriantha': 'Ericales', 'Aegiceras corniculatum': 'Ericales', 'Aegilops tauschii': 'Poales', 'Aeschynomene evenia': 'Fabales', 'Aethionema arabicum': 'Brassicales', 'Akebia trifoliata': 'Ranunculales', 'Allium sativum': 'Asparagales', 'Alternaria oxytropis': 'Pleosporales', 'Amaranthus cruentus': 'Caryophyllales', 'Amaranthus hybridus': 'Caryophyllales', 'Amaranthus hypochondriacus': 'Caryophyllales', 'Amaranthus palmeri': 'Caryophyllales', 'Amaranthus tuberculatus': 'Caryophyllales', 'Amborella trichopoda': 'Amborellales', 'Ammopiptanthus nanus': 'Fabales', 'Amorphophallus konjac': 'Alismatales', 'Amphicarpaea edgeworthii': 'Fabales', 'Ananas comosus': 'Poales', 'Andrographis paniculata': 'Lamiales', 'Angelica sinensis': 'Apiales', 'Antirrhinum majus': 'Lamiales', 'Apium graveolens': 'Apiales', 'Apostasia shenzhenica': 'Asparagales', 'Aquilaria sinensis': 'Malvales', 'Aquilegia coerulea': 'Ranunculales', 'Aquilegia oxysepala': 'Ranunculales', 'Arabidopsis lyrata': 'Brassicales', 'Arabidopsis thaliana': 'Brassicales', 'Arabis alpina': 'Brassicales', 'Arachis duranensis': 'Fabales', 'Arachis hypogaea': 'Fabales', 'Aralia elata': 'Apiales', 'Arctium lappa': 'Asterales', 'Argentina anserina': 'Rosales', 'Aristolochia contorta': 'Piperales', 'Aristolochia fimbriata': 'Piperales', 'Artemisia annua': 'Asterales', 'Artocarpus nanchuanensis': 'Rosales', 'Asparagus officinalis': 'Asparagales', 'Asparagus setaceus': 'Asparagales', 'Atriplex hortensis': 'Caryophyllales', 'Avena insularis': 'Poales', 'Avena sativa': 'Poales', 'Averrhoa carambola': 'Oxalidales', 'Avicennia marina': 'Lamiales', 'Bauhinia variegata': 'Fabales', 'Begonia darthvaderiana': 'Cucurbitales', 'Begonia fuchsioides': 'Cucurbitales', 'Benincasa hispida': 'Cucurbitales', 'Beta patula': 'Caryophyllales', 'Beta vulgaris': 'Caryophyllales', 'Betula pendula': 'Fagales', 'Boechera stricta': 'Brassicales', 'Boehmeria nivea': 'Rosales', 'Bombax ceiba': 'Malvales', 'Bonia amplexicaulis': 'Poales', 'Brachypodium distachyon': 'Poales', 'Brachypodium hybridum': 'Poales', 'Brassica napus': 'Brassicales', 'Brassica oleracea': 'Brassicales', 'Brassica rapa': 'Brassicales', 'Bretschneidera sinensis': 'Brassicales', 'Bruguiera sexangula': 'Malpighiales', 'Buddleja alternifolia': 'Lamiales', 'Buxus austroyunnanensis': 'Buxales', 'Buxus sinica': 'Buxales', 'Cajanus cajan': 'Fabales', 'Calamus simplicifolius': 'Arecales', 'Callicarpa americana': 'Lamiales', 'Calotropis gigantea': 'Gentianales', 'Camelina sativa': 'Brassicales', 'Camellia chekiangoleosa': 'Ericales', 'Camellia oleifera': 'Ericales', 'Camptotheca acuminata': 'Cornales', 'Cannabis sativa': 'Rosales', 'Capparis spinosa': 'Brassicales', 'Capsella rubella': 'Brassicales', 'Capsicum annuum': 'Solanales', 'Capsicum baccatum': 'Solanales', 'Capsicum chinense': 'Solanales', 'Cardamine hirsuta': 'Brassicales', 'Carex cristatella': 'Poales', 'Carex scoparia': 'Poales', 'Carica papaya': 'Brassicales', 'Carpinus fangiana': 'Fagales', 'Carthamus tinctorius': 'Asterales', 'Carya illinoinensis': 'Fagales', 'Castanea mollissima': 'Fagales', 'Casuarina glauca': 'Fagales', 'Catharanthus roseus': 'Gentianales', 'Cephalotus follicularis': 'Oxalidales', 'Ceratophyllum demersum': 'Ceratophyllales', 'Cercis chinensis': 'Fabales', 'Chenopodium quinoa': 'Caryophyllales', 'Chimonanthus salicifolius': 'Laurales', 'Chiococca alba': 'Gentianales', 'Chloranthus sessilifolius': 'Chloranthales', 'Chloranthus spicatus': 'Chloranthales', 'Chrysanthemum seticuspe': 'Asterales', 'Cicer arietinum': 'Fabales', 'Cichorium endivia': 'Asterales', 'Cichorium intybus': 'Asterales', 'Cinchona pubescens': 'Gentianales', 'Cinnamomum kanehirae': 'Laurales', 'Citrullus lanatus': 'Cucurbitales', 'Citrus clementina': 'Sapindales', 'Citrus maxima': 'Sapindales', 'Citrus sinensis': 'Sapindales', 'Cleome violacea': 'Brassicales', 'Clethra arborea': 'Ericales', 'Cocos nucifera': 'Arecales', 'Coffea canephora': 'Gentianales', 'Coffea humblotiana': 'Gentianales', 'Coptis chinensis': 'Ranunculales', 'Corchorus capsularis': 'Malvales', 'Coriandrum sativum': 'Apiales', 'Corydalis tomentella': 'Ranunculales', 'Corylus heterophylla': 'Fagales', 'Crataegus pinnatifida': 'Rosales', 'Cremastra appendiculata': 'Asparagales', 'Cucumis melo': 'Cucurbitales', 'Cucumis sativus': 'Cucurbitales', 'Cucurbita argyrosperma': 'Cucurbitales', 'Cucurbita maxima': 'Cucurbitales', 'Cuscuta campestris': 'Solanales', 'Cycas panzhihuaensis': 'Cycadales', 'Cyclocarya paliurus': 'Fagales', 'Cymodocea nodosa': 'Alismatales', 'Cynara cardunculus': 'Asterales', 'Daemonorops jenkinsiana': 'Arecales', 'Dalbergia odorifera': 'Fabales', 'Datisca glomerata': 'Cucurbitales', 'Datura stramonium': 'Solanales', 'Daucus carota': 'Apiales', 'Davidia involucrata': 'Cornales', 'Dendrobium nobile': 'Asparagales', 'Dianthus caryophyllus': 'Caryophyllales', 'Dichanthelium oligosanthes': 'Poales', 'Dimocarpus longan': 'Sapindales', 'Dioscorea alata': 'Dioscoreales', 'Dioscorea rotundata': 'Dioscoreales', 'Diospyros lotus': 'Ericales', 'Draba nivalis': 'Brassicales', 'Dryas drummondii': 'Rosales', 'Durio zibethinus': 'Malvales', 'Echinochloa crus-galli': 'Poales', 'Elaeagnus angustifolia': 'Rosales', 'Elaeis guineensis': 'Arecales', 'Ensete glaucum': 'Zingiberales', 'Ensete ventricosum': 'Zingiberales', 'Entada phaseoloides': 'Fabales', 'Epimedium pubescens': 'Ranunculales', 'Eragrostis nindensis': 'Poales', 'Eragrostis tef': 'Poales', 'Erigeron canadensis': 'Asterales', 'Eriobotrya japonica': 'Rosales', 'Eruca sativa': 'Brassicales', 'Erysimum cheiranthoides': 'Brassicales', 'Eucalyptus grandis': 'Myrtales', 'Euphorbia lathyris': 'Malpighiales', 'Eupristina verticillata': 'Hymenoptera', 'Euryale ferox': 'Nymphaeales', 'Euscaphis japonica': 'Crossosomatales', 'Eutrema salsugineum': 'Brassicales', 'Fagopyrum tataricum': 'Caryophyllales', 'Fagus sylvatica': 'Fagales', 'Faidherbia albida': 'Fabales', 'Ficus hispida': 'Rosales', 'Ficus microcarpa': 'Rosales', 'Fragaria pentaphylla': 'Rosales', 'Fragaria vesca': 'Rosales', 'Fraxinus pennsylvanica': 'Lamiales', 'Gardenia jasminoides': 'Gentianales', 'Genlisea aurea': 'Lamiales', 'Gilia yorkii': 'Ericales', 'Glycine max': 'Fabales', 'Glycine soja': 'Fabales', 'Gossypium barbadense': 'Malvales', 'Gossypium hirsutum': 'Malvales', 'Gossypium raimondii': 'Malvales', 'Guadua angustifolia': 'Poales', 'Gypsophila paniculata': 'Caryophyllales', 'Haloxylon ammodendron': 'Caryophyllales', 'Handroanthus impetiginosus': 'Lamiales', 'Helianthus annuus': 'Asterales', 'Hevea brasiliensis': 'Malpighiales', 'Hibiscus cannabinus': 'Malvales', 'Hippophae rhamnoides': 'Rosales', 'Hordeum vulgare': 'Poales', 'Humulus lupulus': 'Rosales', 'Hydrangea macrophylla': 'Cornales', 'Hylocereus undatus': 'Caryophyllales', 'Ilex polyneura': 'Aquifoliales', 'Ipomoea nil': 'Solanales', 'Jacaranda mimosifolia': 'Lamiales', 'Jasminum sambac': 'Lamiales', 'Jatropha curcas': 'Malpighiales', 'Juglans mandshurica': 'Fagales', 'Juglans regia': 'Fagales', 'Juglans sigillata': 'Fagales', 'Juncus effusus': 'Poales', 'Juncus inflexus': 'Poales', 'Kalanchoe fedtschenkoi': 'Saxifragales', 'Kandelia candel': 'Malpighiales', 'Kandelia obovata': 'Malpighiales', 'Kingdonia uniflora': 'Ranunculales', 'Lablab purpureus': 'Fabales', 'Lactuca sativa': 'Asterales', 'Leersia perrieri': 'Poales', 'Lemna minor': 'Alismatales', 'Lemna minuta': 'Alismatales', 'Lens culinaris': 'Fabales', 'Lens ervoides': 'Fabales', 'Lepidium sativum': 'Brassicales', 'Lindenbergia luchunensis': 'Lamiales', 'Linum usitatissimum': 'Malpighiales', 'Liriodendron chinense': 'Magnoliales', 'Liriodendron tulipifera': 'Magnoliales', 'Litchi chinensis': 'Sapindales', 'Lithospermum erythrorhizon': 'Boraginales', 'Lobularia maritima': 'Brassicales', 'Iochroma cyaneum': 'Solanales', 'Lonicera japonica': 'Dipsacales', 'Lotus japonicus': 'Fabales', 'Lupinus angustifolius': 'Fabales', 'Macadamia integrifolia': 'Proteales', 'Macadamia jansenii': 'Proteales', 'Macleaya cordata': 'Ranunculales', 'Macrotyloma uniflorum': 'Fabales', 'Magnolia biondii': 'Magnoliales', 'Malania oleifera': 'Santalales', 'Malus domestica': 'Rosales', 'Malus sieversii': 'Rosales', 'Mangifera indica': 'Sapindales', 'Manihot esculenta': 'Malpighiales', 'Medicago sativa': 'Fabales', 'Medicago truncatula': 'Fabales', 'Megacarpaea delavayi': 'Brassicales', 'Megadenia pygmaea': 'Brassicales', 'Melaleuca alternifolia': 'Myrtales', 'Melastoma dodecandrum': 'Myrtales', 'Melilotus albus': 'Fabales', 'Microthlaspi erraticum': 'Brassicales', 'Mikania micrantha': 'Asterales', 'Mimulus guttatus': 'Lamiales', 'Miscanthus lutarioriparius': 'Poales', 'Morella rubra': 'Fagales', 'Moringa oleifera': 'Brassicales', 'Morus notabilis': 'Rosales', 'Mucuna pruriens': 'Fabales', 'Musa acuminata': 'Zingiberales', 'Musa balbisiana': 'Zingiberales', 'Nelumbo nucifera': 'Proteales', 'Nephelium lappaceum': 'Sapindales', 'Nicotiana attenuata': 'Solanales', 'Nicotiana tabacum': 'Solanales', 'Nicotiana tomentosiformis': 'Solanales', 'Nymphaea colorata': 'Nymphaeales', 'Nymphaea thermarum': 'Nymphaeales', 'Nymphoides indica': 'Asterales', 'Nyssa sinensis': 'Cornales', 'Nyssa yunnanensis': 'Cornales', 'Ocimum basilicum': 'Lamiales', 'Olea europaea': 'Lamiales', 'Olyra latifolia': 'Poales', 'Ophiorrhiza pumila': 'Gentianales', 'Origanum majorana': 'Lamiales', 'Origanum vulgare': 'Lamiales', 'Orobanche cumana': 'Lamiales', 'Oropetium thomaeum': 'Poales', 'Orychophragmus violaceus': 'Brassicales', 'Oryza glaberrima': 'Poales', 'Oryza punctata': 'Poales', 'Oryza rufipogon': 'Poales', 'Oryza sativa': 'Poales', 'Oxytropis ochrocephala': 'Fabales', 'Paeonia ostii': 'Saxifragales', 'Paeonia suffruticosa': 'Saxifragales', 'Panax ginseng': 'Apiales', 'Panax japonicus': 'Apiales', 'Panax quinquefolius': 'Apiales', 'Panax stipuleanatus': 'Apiales', 'Panicum virgatum': 'Poales', 'Papaver rhoeas': 'Ranunculales', 'Papaver setigerum': 'Ranunculales', 'Papaver somniferum': 'Ranunculales', 'Parasponia andersonii': 'Rosales', 'Passiflora edulis': 'Malpighiales', 'Passiflora organensis': 'Malpighiales', 'Paulownia fortunei': 'Lamiales', 'Pennisetum glaucum': 'Poales', 'Perilla frutescens': 'Lamiales', 'Persea americana': 'Laurales', 'Petunia axillaris': 'Solanales', 'Phalaenopsis equestris': 'Asparagales', 'Pharus latifolius': 'Poales', 'Phaseolus vulgaris': 'Fabales', 'Phelipanche aegyptiaca': 'Lamiales', 'Phoenix dactylifera': 'Arecales', 'Phragmites australis': 'Poales', 'Phtheirospermum japonicum': 'Lamiales', 'Physalis grisea': 'Solanales', 'Physalis pruinosa': 'Solanales', 'Picrorhiza kurrooa': 'Lamiales', 'Piper nigrum': 'Piperales', 'Pistacia vera': 'Sapindales', 'Pistia stratiotes': 'Alismatales', 'Pisum sativm': 'Fabales', 'Platycarya strobilacea': 'Fagales', 'Pogostemon cablin': 'Lamiales', 'Poncirus trifoliata': 'Sapindales', 'Populus trichocarpa': 'Malpighiales', 'Populus wilsonii': 'Malpighiales', 'Portulaca amilis': 'Caryophyllales', 'Posidonia oceanica': 'Alismatales', 'Potamogeton acutifolius': 'Alismatales', 'Primula veris': 'Ericales', 'Protea cynaroides': 'Proteales', 'Prunus mume': 'Rosales', 'Prunus persica': 'Rosales', 'Punica granatum': 'Myrtales', 'Pyrus betulifolia': 'Rosales', 'Pyrus bretschneideri': 'Rosales', 'Quercus acutissima': 'Fagales', 'Quercus robur': 'Fagales', 'Quercus rubra': 'Fagales', 'Raddia guianensis': 'Poales', 'Raphanus sativus': 'Brassicales', 'Rhizophora apiculata': 'Malpighiales', 'Rhodiola crenulata': 'Saxifragales', 'Rhododendron griersonianum': 'Ericales', 'Rhododendron simsii': 'Ericales', 'Rhynchospora breviuscula': 'Poales', 'Rhynchospora pubera': 'Poales', 'Rhynchospora tenuis': 'Poales', 'Ricinus communis': 'Malpighiales', 'Roridula gorgonias': 'Ericales', 'Rosa chinensis': 'Rosales', 'Rosa rugosa': 'Rosales', 'Rosmarinus officinalis': 'Lamiales', 'Rubus corchorifolius': 'Rosales', 'Rubus occidentalis': 'Rosales', 'Saccharum officinarum': 'Poales', 'Salix arbutifolia': 'Malpighiales', 'Salix chaenomeloides': 'Malpighiales', 'Salvia splendens': 'Lamiales', 'Sapindus mukorossi': 'Sapindales', 'Schrenkiella parvula': 'Brassicales', 'Sclerocarya birrea': 'Sapindales', 'Scutellaria baicalensis': 'Lamiales', 'Secale cereale': 'Poales', 'Sedum album': 'Saxifragales', 'Senna tora': 'Fabales', 'Sesamum indicum': 'Lamiales', 'Setaria italica': 'Poales', 'Setaria viridis': 'Poales', 'Shorea leprosula': 'Malvales', 'Simmondsia chinensis': 'Caryophyllales', 'Sinapis alba': 'Brassicales', 'Siraitia grosvenorii': 'Cucurbitales', 'Smallanthus sonchifolius': 'Asterales', 'Solanum lycopersicum': 'Solanales', 'Solanum pennellii': 'Solanales', 'Solanum tuberosum': 'Solanales', 'Sonneratia alba': 'Myrtales', 'Sonneratia caseolaris': 'Myrtales', 'Sorbus pohuashanensis': 'Rosales', 'Sorghum bicolor': 'Poales', 'Spatholobus suberectus': 'Fabales', 'Spinacia oleracea': 'Caryophyllales', 'Spirodela intermedia': 'Alismatales', 'Spirodela polyrhiza': 'Alismatales', 'Stellera chamaejasme': 'Malvales', 'Stevia rebaudiana': 'Asterales', 'Streptocarpus rexii': 'Lamiales', 'Streptochaeta angustifolia': 'Poales', 'Striga asiatica': 'Lamiales', 'Striga hermonthica': 'Lamiales', 'Strobilanthes cusia': 'Lamiales', 'Suaeda aralocaspica': 'Caryophyllales', 'Syringa oblata': 'Lamiales', 'Syzygium aromaticum': 'Myrtales', 'Taraxacum kok-saghyz': 'Asterales', 'Taraxacum mongolicum': 'Asterales', 'Tarenaya hassleriana': 'Brassicales', 'Tectona grandis': 'Lamiales', 'Telopea speciosissima': 'Proteales', 'Tetracentron sinense': 'Trochodendrales', 'Thalassia testudinum': 'Alismatales', 'Thalictrum thalictroides': 'Ranunculales', 'Thellungiella parvula': 'Brassicales', 'Theobroma cacao': 'Malvales', 'Thinopyrum intermedium': 'Poales', 'Thlaspi arvense': 'Brassicales', 'Thymus quinquecostatus': 'Lamiales', 'Toona sinensis': 'Sapindales', 'Trema orientale': 'Rosales', 'Trichopus zeylanicus': 'Dioscoreales', 'Trifolium pratense': 'Fabales', 'Trifolium subterraneum': 'Fabales', 'Tripterygium wilfordii': 'Celastrales', 'Triticum aestivum': 'Poales', 'Triticum turgidum': 'Poales', 'Trochodendron aralioides': 'Trochodendrales', 'Utricularia gibba': 'Lamiales', 'Vaccinium caesariense': 'Ericales', 'Vaccinium corymbosum': 'Ericales', 'Vernicia fordii': 'Malpighiales', 'Vicia sativa': 'Fabales', 'Vigna angularis': 'Fabales', 'Vigna radiata': 'Fabales', 'Vigna subterranea': 'Fabales', 'Vitellaria paradoxa': 'Ericales', 'Vitis arizonica': 'Vitales', 'Vitis vinifera': 'Vitales', 'Wolffia australiana': 'Alismatales', 'Wurfbainia villosa': 'Zingiberales', 'Xanthoceras sorbifolium': 'Sapindales', 'Zanthoxylum armatum': 'Sapindales', 'Zanthoxylum bungeanum': 'Sapindales', 'Zea mays': 'Poales', 'Zingiber officinale': 'Zingiberales', 'Zizania latifolia': 'Poales', 'Zizania palustris': 'Poales', 'Ziziphus jujuba var.spinosa': 'Rosales', 'Zostera marina': 'Alismatales', 'Zostera muelleri': 'Alismatales', 'Zoysia japonica': 'Poales', 'Zoysia matrella': 'Poales'}

Species_Completeness = {'Abrus precatorius': 0.992, 'Acacia pycnantha': 0.9740000000000001, 'Acanthochlamys bracteata': 0.95, 'Acer pseudosieboldianum': 0.986, 'Acer yangbiense': 0.929, 'Achnatherum splendens': 0.852, 'Acorus americanus': 0.972, 'Acorus tatarinowii': 0.948, 'Actinidia chinensis': 0.878, 'Actinidia eriantha': 0.8390000000000001, 'Aegiceras corniculatum': 0.897, 'Aegilops tauschii': 0.993, 'Aeschynomene evenia': 0.9690000000000001, 'Aethionema arabicum': 0.971, 'Akebia trifoliata': 0.9590000000000001, 'Allium sativum': 0.8340000000000001, 'Alternaria oxytropis': 0.076, 'Amaranthus cruentus': 0.907, 'Amaranthus hybridus': 0.736, 'Amaranthus hypochondriacus': 0.804, 'Amaranthus palmeri': 0.695, 'Amaranthus tuberculatus': 0.649, 'Amborella trichopoda': 0.983, 'Ammopiptanthus nanus': 0.972, 'Amorphophallus konjac': 0.905, 'Amphicarpaea edgeworthii': 0.9470000000000001, 'Ananas comosus': 0.894, 'Andrographis paniculata': 0.988, 'Angelica sinensis': 0.878, 'Antirrhinum majus': 0.945, 'Apium graveolens': 0.807, 'Apostasia shenzhenica': 0.812, 'Aquilaria sinensis': 0.938, 'Aquilegia coerulea': 0.98, 'Aquilegia oxysepala': 0.958, 'Arabidopsis lyrata': 0.9790000000000001, 'Arabidopsis thaliana': 0.993, 'Arabis alpina': 0.7959999999999999, 'Arachis duranensis': 0.887, 'Arachis hypogaea': 0.9570000000000001, 'Aralia elata': 0.973, 'Arctium lappa': 0.97, 'Argentina anserina': 0.816, 'Aristolochia contorta': 0.96, 'Aristolochia fimbriata': 0.97, 'Artemisia annua': 0.91, 'Artocarpus nanchuanensis': 0.9840000000000001, 'Asparagus officinalis': 0.7829999999999999, 'Asparagus setaceus': 0.853, 'Atriplex hortensis': 0.867, 'Avena insularis': 0.986, 'Avena sativa': 0.99, 'Averrhoa carambola': 0.9279999999999999, 'Avicennia marina': 0.9690000000000001, 'Bauhinia variegata': 0.9890000000000001, 'Begonia darthvaderiana': 0.754, 'Begonia fuchsioides': 0.9470000000000001, 'Benincasa hispida': 0.99, 'Beta patula': 0.95, 'Beta vulgaris': 0.968, 'Betula pendula': 0.9470000000000001, 'Boechera stricta': 0.966, 'Boehmeria nivea': 0.927, 'Bombax ceiba': 0.961, 'Bonia amplexicaulis': 0.812, 'Brachypodium distachyon': 0.995, 'Brachypodium hybridum': 1.0, 'Brassica napus': 0.883, 'Brassica oleracea': 0.785, 'Brassica rapa': 0.966, 'Bretschneidera sinensis': 0.9790000000000001, 'Bruguiera sexangula': 0.929, 'Buddleja alternifolia': 0.927, 'Buxus austroyunnanensis': 0.943, 'Buxus sinica': 0.767, 'Cajanus cajan': 0.938, 'Calamus simplicifolius': 0.785, 'Callicarpa americana': 0.9420000000000001, 'Calotropis gigantea': 0.919, 'Camelina sativa': 0.992, 'Camellia chekiangoleosa': 0.879, 'Camellia oleifera': 0.5329999999999999, 'Camptotheca acuminata': 0.9490000000000001, 'Cannabis sativa': 0.919, 'Capparis spinosa': 0.851, 'Capsella rubella': 0.9740000000000001, 'Capsicum annuum': 0.863, 'Capsicum baccatum': 0.813, 'Capsicum chinense': 0.797, 'Cardamine hirsuta': 0.9620000000000001, 'Carex cristatella': 0.841, 'Carex scoparia': 0.863, 'Carica papaya': 0.948, 'Carpinus fangiana': 0.841, 'Carthamus tinctorius': 0.9109999999999999, 'Carya illinoinensis': 0.985, 'Castanea mollissima': 0.971, 'Casuarina glauca': 0.9520000000000001, 'Catharanthus roseus': 0.7829999999999999, 'Cephalotus follicularis': 0.9670000000000001, 'Ceratophyllum demersum': 0.715, 'Cercis chinensis': 0.935, 'Chenopodium quinoa': 0.948, 'Chimonanthus salicifolius': 0.868, 'Chiococca alba': 0.956, 'Chloranthus sessilifolius': 0.828, 'Chloranthus spicatus': 0.8540000000000001, 'Chrysanthemum seticuspe': 0.9009999999999999, 'Cicer arietinum': 0.904, 'Cichorium endivia': 0.975, 'Cichorium intybus': 0.966, 'Cinchona pubescens': 0.57, 'Cinnamomum kanehirae': 0.897, 'Citrullus lanatus': 0.9670000000000001, 'Citrus clementina': 0.9540000000000001, 'Citrus maxima': 0.9540000000000001, 'Citrus sinensis': 0.867, 'Cleome violacea': 0.993, 'Clethra arborea': 0.8809999999999999, 'Cocos nucifera': 0.7, 'Coffea canephora': 0.941, 'Coffea humblotiana': 0.7859999999999999, 'Coptis chinensis': 0.79, 'Corchorus capsularis': 0.807, 'Coriandrum sativum': 0.909, 'Corydalis tomentella': 0.914, 'Corylus heterophylla': 0.9209999999999999, 'Crataegus pinnatifida': 0.9640000000000001, 'Cremastra appendiculata': 0.87, 'Cucumis melo': 0.938, 'Cucumis sativus': 0.953, 'Cucurbita argyrosperma': 0.971, 'Cucurbita maxima': 0.9620000000000001, 'Cuscuta campestris': 0.8140000000000001, 'Cycas panzhihuaensis': 0.89, 'Cyclocarya paliurus': 0.61, 'Cymodocea nodosa': 0.9570000000000001, 'Cynara cardunculus': 0.9890000000000001, 'Daemonorops jenkinsiana': 0.818, 'Dalbergia odorifera': 0.9209999999999999, 'Datisca glomerata': 0.893, 'Datura stramonium': 0.853, 'Daucus carota': 0.855, 'Davidia involucrata': 0.907, 'Dendrobium nobile': 0.934, 'Dianthus caryophyllus': 0.823, 'Dichanthelium oligosanthes': 0.687, 'Dimocarpus longan': 0.897, 'Dioscorea alata': 0.971, 'Dioscorea rotundata': 0.765, 'Diospyros lotus': 0.68, 'Draba nivalis': 0.96, 'Dryas drummondii': 0.909, 'Durio zibethinus': 0.987, 'Echinochloa crus-galli': 0.977, 'Elaeagnus angustifolia': 0.938, 'Elaeis guineensis': 0.9740000000000001, 'Ensete glaucum': 0.948, 'Ensete ventricosum': 0.252, 'Entada phaseoloides': 0.89, 'Epimedium pubescens': 0.858, 'Eragrostis nindensis': 0.9279999999999999, 'Eragrostis tef': 0.976, 'Erigeron canadensis': 0.99, 'Eriobotrya japonica': 0.992, 'Eruca sativa': 0.983, 'Erysimum cheiranthoides': 0.977, 'Eucalyptus grandis': 0.9390000000000001, 'Euphorbia lathyris': 0.91, 'Eupristina verticillata': 0.109, 'Euryale ferox': 0.948, 'Euscaphis japonica': 0.965, 'Eutrema salsugineum': 0.987, 'Fagopyrum tataricum': 0.893, 'Fagus sylvatica': 0.758, 'Faidherbia albida': 0.95, 'Ficus hispida': 0.9309999999999999, 'Ficus microcarpa': 0.91, 'Fragaria pentaphylla': 0.7959999999999999, 'Fragaria vesca': 0.987, 'Fraxinus pennsylvanica': 0.82, 'Gardenia jasminoides': 0.871, 'Genlisea aurea': 0.701, 'Gilia yorkii': 0.8440000000000001, 'Glycine max': 0.995, 'Glycine soja': 0.987, 'Gossypium barbadense': 0.998, 'Gossypium hirsutum': 0.997, 'Gossypium raimondii': 0.986, 'Guadua angustifolia': 0.433, 'Gypsophila paniculata': 0.907, 'Haloxylon ammodendron': 0.945, 'Handroanthus impetiginosus': 0.7909999999999999, 'Helianthus annuus': 0.966, 'Hevea brasiliensis': 0.68, 'Hibiscus cannabinus': 0.9840000000000001, 'Hippophae rhamnoides': 0.9259999999999999, 'Hordeum vulgare': 0.9390000000000001, 'Humulus lupulus': 0.725, 'Hydrangea macrophylla': 0.784, 'Hylocereus undatus': 0.9129999999999999, 'Ilex polyneura': 0.9520000000000001, 'Ipomoea nil': 0.993, 'Jacaranda mimosifolia': 0.92, 'Jasminum sambac': 0.887, 'Jatropha curcas': 0.899, 'Juglans mandshurica': 0.9540000000000001, 'Juglans regia': 0.9640000000000001, 'Juglans sigillata': 0.943, 'Juncus effusus': 0.8740000000000001, 'Juncus inflexus': 0.743, 'Kalanchoe fedtschenkoi': 0.925, 'Kandelia candel': 0.8490000000000001, 'Kandelia obovata': 0.8740000000000001, 'Kingdonia uniflora': 0.76, 'Lablab purpureus': 0.775, 'Lactuca sativa': 0.9590000000000001, 'Leersia perrieri': 0.919, 'Lemna minor': 0.469, 'Lemna minuta': 0.31, 'Lens culinaris': 0.943, 'Lens ervoides': 0.958, 'Lepidium sativum': 0.9890000000000001, 'Lindenbergia luchunensis': 0.975, 'Linum usitatissimum': 0.951, 'Liriodendron chinense': 0.747, 'Liriodendron tulipifera': 0.9470000000000001, 'Litchi chinensis': 0.9359999999999999, 'Lithospermum erythrorhizon': 0.78, 'Lobularia maritima': 0.971, 'Iochroma cyaneum': 0.9179999999999999, 'Lonicera japonica': 0.8190000000000001, 'Lotus japonicus': 0.992, 'Lupinus angustifolius': 0.992, 'Macadamia integrifolia': 0.929, 'Macadamia jansenii': 0.654, 'Macleaya cordata': 0.93, 'Macrotyloma uniflorum': 0.9209999999999999, 'Magnolia biondii': 0.7120000000000001, 'Malania oleifera': 0.731, 'Malus domestica': 0.966, 'Malus sieversii': 0.9640000000000001, 'Mangifera indica': 0.997, 'Manihot esculenta': 0.997, 'Medicago sativa': 0.7809999999999999, 'Medicago truncatula': 0.966, 'Megacarpaea delavayi': 0.9279999999999999, 'Megadenia pygmaea': 0.956, 'Melaleuca alternifolia': 0.902, 'Melastoma dodecandrum': 0.8909999999999999, 'Melilotus albus': 0.9640000000000001, 'Microthlaspi erraticum': 0.988, 'Mikania micrantha': 0.885, 'Mimulus guttatus': 0.9229999999999999, 'Miscanthus lutarioriparius': 0.9690000000000001, 'Morella rubra': 0.804, 'Moringa oleifera': 0.8340000000000001, 'Morus notabilis': 0.975, 'Mucuna pruriens': 0.986, 'Musa acuminata': 0.847, 'Musa balbisiana': 0.871, 'Nelumbo nucifera': 0.99, 'Nephelium lappaceum': 0.799, 'Nicotiana attenuata': 0.992, 'Nicotiana tabacum': 0.972, 'Nicotiana tomentosiformis': 0.96, 'Nymphaea colorata': 0.966, 'Nymphaea thermarum': 0.795, 'Nymphoides indica': 0.912, 'Nyssa sinensis': 0.8959999999999999, 'Nyssa yunnanensis': 0.545, 'Ocimum basilicum': 0.9009999999999999, 'Olea europaea': 0.738, 'Olyra latifolia': 0.883, 'Ophiorrhiza pumila': 0.893, 'Origanum majorana': 0.9229999999999999, 'Origanum vulgare': 0.9179999999999999, 'Orobanche cumana': 0.807, 'Oropetium thomaeum': 0.721, 'Orychophragmus violaceus': 0.961, 'Oryza glaberrima': 0.848, 'Oryza punctata': 0.9359999999999999, 'Oryza rufipogon': 0.9440000000000001, 'Oryza sativa': 0.915, 'Oxytropis ochrocephala': 0.961, 'Paeonia ostii': 0.787, 'Paeonia suffruticosa': 0.326, 'Panax ginseng': 0.976, 'Panax japonicus': 0.948, 'Panax quinquefolius': 0.9520000000000001, 'Panax stipuleanatus': 0.8740000000000001, 'Panicum virgatum': 0.997, 'Papaver rhoeas': 0.826, 'Papaver setigerum': 0.96, 'Papaver somniferum': 0.914, 'Parasponia andersonii': 0.9279999999999999, 'Passiflora edulis': 0.825, 'Passiflora organensis': 0.9490000000000001, 'Paulownia fortunei': 0.975, 'Pennisetum glaucum': 0.782, 'Perilla frutescens': 0.861, 'Persea americana': 0.687, 'Petunia axillaris': 0.9129999999999999, 'Phalaenopsis equestris': 0.9129999999999999, 'Pharus latifolius': 0.92, 'Phaseolus vulgaris': 0.9790000000000001, 'Phelipanche aegyptiaca': 0.755, 'Phoenix dactylifera': 0.982, 'Phragmites australis': 0.879, 'Phtheirospermum japonicum': 0.851, 'Physalis grisea': 0.971, 'Physalis pruinosa': 0.971, 'Picrorhiza kurrooa': 0.348, 'Piper nigrum': 0.825, 'Pistacia vera': 0.963, 'Pistia stratiotes': 0.856, 'Pisum sativm': 0.986, 'Platycarya strobilacea': 0.937, 'Pogostemon cablin': 0.993, 'Poncirus trifoliata': 0.987, 'Populus trichocarpa': 0.991, 'Populus wilsonii': 0.965, 'Portulaca amilis': 0.915, 'Posidonia oceanica': 0.9690000000000001, 'Potamogeton acutifolius': 0.9740000000000001, 'Primula veris': 0.9309999999999999, 'Protea cynaroides': 0.9790000000000001, 'Prunus mume': 0.986, 'Prunus persica': 0.992, 'Punica granatum': 0.961, 'Pyrus betulifolia': 0.943, 'Pyrus bretschneideri': 0.879, 'Quercus acutissima': 0.8490000000000001, 'Quercus robur': 0.519, 'Quercus rubra': 0.976, 'Raddia guianensis': 0.57, 'Raphanus sativus': 0.894, 'Rhizophora apiculata': 0.927, 'Rhodiola crenulata': 0.823, 'Rhododendron griersonianum': 0.909, 'Rhododendron simsii': 0.9, 'Rhynchospora breviuscula': 0.882, 'Rhynchospora pubera': 0.922, 'Rhynchospora tenuis': 0.8690000000000001, 'Ricinus communis': 0.9159999999999999, 'Roridula gorgonias': 0.813, 'Rosa chinensis': 0.93, 'Rosa rugosa': 0.946, 'Rosmarinus officinalis': 0.915, 'Rubus corchorifolius': 0.9129999999999999, 'Rubus occidentalis': 0.9420000000000001, 'Saccharum officinarum': 0.883, 'Salix arbutifolia': 0.919, 'Salix chaenomeloides': 0.9440000000000001, 'Salvia splendens': 0.991, 'Sapindus mukorossi': 0.723, 'Schrenkiella parvula': 0.96, 'Sclerocarya birrea': 0.812, 'Scutellaria baicalensis': 0.862, 'Secale cereale': 0.983, 'Sedum album': 0.866, 'Senna tora': 0.9009999999999999, 'Sesamum indicum': 0.987, 'Setaria italica': 0.981, 'Setaria viridis': 0.9940000000000001, 'Shorea leprosula': 0.885, 'Simmondsia chinensis': 0.912, 'Sinapis alba': 0.9840000000000001, 'Siraitia grosvenorii': 0.77, 'Smallanthus sonchifolius': 0.992, 'Solanum lycopersicum': 0.912, 'Solanum pennellii': 0.991, 'Solanum tuberosum': 0.927, 'Sonneratia alba': 0.927, 'Sonneratia caseolaris': 0.948, 'Sorbus pohuashanensis': 0.968, 'Sorghum bicolor': 0.983, 'Spatholobus suberectus': 0.899, 'Spinacia oleracea': 0.941, 'Spirodela intermedia': 0.9540000000000001, 'Spirodela polyrhiza': 0.813, 'Stellera chamaejasme': 0.938, 'Stevia rebaudiana': 0.9540000000000001, 'Streptocarpus rexii': 0.7440000000000001, 'Streptochaeta angustifolia': 0.851, 'Striga asiatica': 0.9179999999999999, 'Striga hermonthica': 0.9059999999999999, 'Strobilanthes cusia': 0.9790000000000001, 'Suaeda aralocaspica': 0.9079999999999999, 'Syringa oblata': 0.8859999999999999, 'Syzygium aromaticum': 0.945, 'Taraxacum kok-saghyz': 0.765, 'Taraxacum mongolicum': 0.706, 'Tarenaya hassleriana': 0.9940000000000001, 'Tectona grandis': 0.9259999999999999, 'Telopea speciosissima': 0.9840000000000001, 'Tetracentron sinense': 0.6679999999999999, 'Thalassia testudinum': 0.938, 'Thalictrum thalictroides': 0.865, 'Thellungiella parvula': 0.96, 'Theobroma cacao': 0.9940000000000001, 'Thinopyrum intermedium': 0.997, 'Thlaspi arvense': 0.882, 'Thymus quinquecostatus': 0.975, 'Toona sinensis': 0.915, 'Trema orientale': 0.9229999999999999, 'Trichopus zeylanicus': 0.578, 'Trifolium pratense': 0.8859999999999999, 'Trifolium subterraneum': 0.9259999999999999, 'Tripterygium wilfordii': 0.995, 'Triticum aestivum': 0.993, 'Triticum turgidum': 0.9470000000000001, 'Trochodendron aralioides': 0.892, 'Utricularia gibba': 0.745, 'Vaccinium caesariense': 0.977, 'Vaccinium corymbosum': 0.983, 'Vernicia fordii': 0.955, 'Vicia sativa': 0.9670000000000001, 'Vigna angularis': 0.993, 'Vigna radiata': 0.991, 'Vigna subterranea': 0.9359999999999999, 'Vitellaria paradoxa': 0.851, 'Vitis arizonica': 0.8320000000000001, 'Vitis vinifera': 0.935, 'Wolffia australiana': 0.893, 'Wurfbainia villosa': 0.951, 'Xanthoceras sorbifolium': 0.753, 'Zanthoxylum armatum': 0.915, 'Zanthoxylum bungeanum': 0.9840000000000001, 'Zea mays': 0.93, 'Zingiber officinale': 0.9890000000000001, 'Zizania latifolia': 0.981, 'Zizania palustris': 0.848, 'Ziziphus jujuba var.spinosa': 0.986, 'Zostera marina': 0.9329999999999999, 'Zostera muelleri': 0.6990000000000001, 'Zoysia japonica': 0.374, 'Zoysia matrella': 0.40399999999999997}

Orders_replace_Species = {'Acorales':'(Acorus_americanus,Acorus_tatarinowii)','Alismatales':'(Potamogeton_acutifolius,(Spirodela_intermedia,Amorphophallus_konjac))','Zingiberales':'(Ensete_glaucum,(Zingiber_officinale,Wurfbainia_villosa))','Vitales':'(Vitis_arizonica,Vitis_vinifera)','Trochodendrales':'(Tetracentron_sinense,Trochodendron_aralioides)','Solanales':'(Ipomoea_nil,(Nicotiana_attenuata,Solanum_pennellii))','Saxifragales':'(Paeonia_ostii,(Sedum_album,Kalanchoe_fedtschenkoi))','Sapindales':'(Poncirus_trifoliata,(Mangifera_indica,Pistacia_vera))','Santalales':'Malania_oleifera','Rosales':'(Artocarpus_nanchuanensis,(Eriobotrya_japonica,Prunus_persica))','Ranunculales':'(Papaver_setigerum,(Aquilegia_coerulea,Aquilegia_oxysepala))','Proteales':'(Nelumbo_nucifera,(Telopea_speciosissima,Protea_cynaroides))','Poales':'(Rhynchospora_pubera,(Brachypodium_hybridum,Panicum_virgatum))','Pleosporales':'Alternaria_oxytropis','Piperales':'(Piper_nigrum,(Aristolochia_fimbriata,Aristolochia_contorta))','Pandanales':'Acanthochlamys_bracteata','Oxalidales':'(Averrhoa_carambola,Cephalotus_follicularis)','Nymphaeales':'(Euryale_ferox,(Nymphaea_colorata,Nymphaea_thermarum))','Myrtales':'(Punica_granatum,(Syzygium_aromaticum,Eucalyptus_grandis))','Malvales':'(Aquilaria_sinensis,(Gossypium_hirsutum,Gossypium_barbadense))','Malpighiales':'(Populus_trichocarpa,(Manihot_esculenta,Vernicia_fordii))','Laurales':'(Chimonanthus_salicifolius,(Cinnamomum_kanehirae,Persea_americana))','Lamiales':'(Andrographis_paniculata,(Pogostemon_cablin,Salvia_splendens))','Gentianales':'(Calotropis_gigantea,(Chiococca_alba,Coffea_canephora))','Fagales':'(Quercus_rubra,(Carya_illinoinensis,Juglans_regia))','Fabales':'(Glycine_max,(Vigna_angularis,Vigna_radiata))','Ericales':'(Primula_veris,(Vaccinium_corymbosum,Vaccinium_caesariense))','Dipsacales':'Lonicera_japonica','Dioscoreales':'(Dioscorea_alata,Dioscorea_rotundata)','Cucurbitales':'(Begonia_fuchsioides,(Benincasa_hispida,Cucurbita_argyrosperma))','Cornales':'(Hydrangea_macrophylla,(Camptotheca_acuminata,Davidia_involucrata))','Chloranthales':'(Chloranthus_spicatus,Chloranthus_sessilifolius)','Ceratophyllales':'Ceratophyllum_demersum','Celastrales':'Tripterygium_wilfordii','Caryophyllales':'(Portulaca_amilis,(Beta_vulgaris,Beta_patula))','Buxales':'(Buxus_austroyunnanensis,Buxus_sinica)','Brassicales':'(Arabidopsis_thaliana,(Tarenaya_hassleriana,Cleome_violacea))','Boraginales':'Lithospermum_erythrorhizon','Asterales':'(Nymphoides_indica,(Smallanthus_sonchifolius,Erigeron_canadensis))','Asparagales':'(Asparagus_setaceus,(Phalaenopsis_equestris,Dendrobium_nobile))','Arecales':'(Daemonorops_jenkinsiana,(Phoenix_dactylifera,Elaeis_guineensis))','Aquifoliales':'Ilex_polyneura','Apiales':'(Coriandrum_sativum,(Aralia_elata,Panax_ginseng))','Amborellales':'Amborella_trichopoda','Magnoliales':'(Liriodendron_chinense,Magnolia_biondii)','Crossosomatales':'Euscaphis_japonica'}

Orders_replace_One_Species = {'Acorales':'Acorus_americanus','Alismatales':'Potamogeton_acutifolius','Zingiberales':'Ensete_glaucum','Vitales':'Vitis_arizonica','Trochodendrales':'Tetracentron_sinense','Solanales':'Ipomoea_nil','Saxifragales':'Paeonia_ostii','Sapindales':'Poncirus_trifoliata','Santalales':'Malania_oleifera','Rosales':'Artocarpus_nanchuanensis','Ranunculales':'Papaver_setigerum','Proteales':'Nelumbo_nucifera','Poales':'Rhynchospora_pubera','Pleosporales':'Alternaria_oxytropis','Piperales':'Piper_nigrum','Pandanales':'Acanthochlamys_bracteata','Oxalidales':'Averrhoa_carambola','Nymphaeales':'Euryale_ferox','Myrtales':'Punica_granatum','Malvales':'Aquilaria_sinensis','Malpighiales':'Populus_trichocarpa','Laurales':'Chimonanthus_salicifolius','Lamiales':'Andrographis_paniculata','Gentianales':'Calotropis_gigantea','Fagales':'Quercus_rubra','Fabales':'Glycine_max','Ericales':'Primula_veris','Dipsacales':'Lonicera_japonica','Dioscoreales':'Dioscorea_alata','Cucurbitales':'Begonia_fuchsioides','Cornales':'Hydrangea_macrophylla','Chloranthales':'Chloranthus_spicatus','Ceratophyllales':'Ceratophyllum_demersum','Celastrales':'Tripterygium_wilfordii','Caryophyllales':'Portulaca_amilis','Buxales':'Buxus_austroyunnanensis','Brassicales':'Arabidopsis_thaliana','Boraginales':'Lithospermum_erythrorhizon','Asterales':'Nymphoides_indica','Asparagales':'Asparagus_setaceus','Arecales':'Daemonorops_jenkinsiana','Aquifoliales':'Ilex_polyneura','Apiales':'Coriandrum_sativum','Amborellales':'Amborella_trichopoda','Magnoliales':'Liriodendron_chinense','Crossosomatales':'Euscaphis_japonica'}

def standalonetaxonomy(sp):
    taxoinfo = get_taxonomy(sp,Fullinfo=True)
    if taxoinfo != None:
        for info in taxoinfo:
            logging.info("{}: {}\n".format(info[0],info[1]))

def get_taxonomy(sp,Fullinfo=False):
    sp = sp.replace("_"," ")
    Entrez.email = 'hengchi.chen@ugent.be'
    handle = Entrez.esearch(db='taxonomy', term=sp)
    record = Entrez.read(handle)
    if record['Count'] != '0':
        # the TaxId is the seed of searching
        taxon_id = record['IdList'][0]
        handle = Entrez.efetch(db='taxonomy', id=taxon_id, retmode='xml')
        record = Entrez.read(handle)
        record[0]['LineageEx']
        taxoinfo = gettaxinfo(record[0]['LineageEx'],Fullinfo)
        return taxoinfo
    elif " " in sp:
        return get_taxonomy(sp.split(" ")[0])
    else:
        logging.error("Coundn't find the taxonomy information of {} on NCBI taxonomy database".format(sp))
        return None

def gettaxinfo(infolist,Fullinfo):
    if Fullinfo:
        taxoinfo = []
        for infodic in infolist: taxoinfo.append((infodic['Rank'],infodic['ScientificName']))
    else:
        genus,family,order,subphylum,phylum,kingdom,superkingdom = '','','','','','',''
        for infodic in infolist:
            if infodic['Rank'] == 'genus': genus = infodic['ScientificName']
            if infodic['Rank'] == 'family': family = infodic['ScientificName']
            if infodic['Rank'] == 'order': order = infodic['ScientificName']
            if infodic['Rank'] == 'subphylum': subphylum = infodic['ScientificName']
            if infodic['Rank'] == 'phylum': phylum = infodic['ScientificName']
            if infodic['Rank'] == 'kingdom': kingdom = infodic['ScientificName']
            if infodic['Rank'] == 'superkingdom': superkingdom = infodic['ScientificName']
        taxoinfo = {'genus':genus,'family':family,'order':order,'subphylum':subphylum,'phylum':phylum,'kingdom':kingdom,'superkingdom':superkingdom}
    return taxoinfo

def findupper(tree):
    Age_Reference = {'Angiospermae':'https://doi.org/10.1073/pnas.1719588115'}
    Age_justification = {'Angiospermae':'The maximum age constraint is based on sediments devoid of angiosperm-like pollen below their first report in the Middle Triassic.'}

def cooccurancerecords(Rocks):
    """
    Fossils cooccurance information container
    """
    y = lambda x,v,b,z:{'Clade':x,'Order':v,'Family':b,'Species':z}
    Sediments_map = {0:'Well Khx-1, Muglad Basin, Sample Kh-17, Cretaceous to of Sudan',1:'West Tiba-1, Zone PII, Sample 12, Cretaceous of Egypt',2:'M3 - San Ramon section, Cretaceous of Argentina',3:'Estancia San José, Cretaceous of Argentina',4:'Crato Formation, Cretaceous of Brazil',5:'Puddledock locality, Cretaceous to of the United States',6:'Kokhav 2 Well, Core 3, Cretaceous to of Israel',7:'Mohgaon Kalan, Paleogene of India',8:'Neuse River Cut-Off, Cretaceous of the United States',9:'Jianshangou bed, near Chaomidian, Cretaceous to of China',10:'Bajo Comisión - Rio Mayer Formation, Cretaceous of Argentina',11:'F45/f0394, middle Gore Lignite Measure, Newvale Mine, Oligocene to of New Zealand',12:'Dakhla Oasis Area, Sh-1a, Cretaceous of Egypt',13:'Old Crossman Clay Pit, Cretaceous of the United States',14:'Florissant Flora, Eocene of the United States',15:'Belvidere, Cretaceous of the United States',16:'Vale de Agua 320, Cretaceous to of Portugal',17:'Bank near Brooke, Cretaceous to of the United States',18:'Del Puerto Canyon, Cretaceous of the United States',19:'Borehole 1-QS-1-MA, Sample 14, Cretaceous to of Brazil',20:'CA 5 - 1982, Cretaceous to of Chile',21:'Wotoushan Formation (Jiaguan Formation), the Chishui area, Guizhou Province, Cretaceous of ChinaWotoushan Formation (Jiaguan Formation), the Chishui area, Guizhou Province, Cretaceous of China',22:'The London Clay Flora - Herne Bay, Eocene of the United Kingdom',23:'Burmese amber, Hkamti amber (QUST collection), Cretaceous of Myanmar',24:'Upper Belcourt Ridge, Cretaceous of Canada',25:'Protection 3826, Cretaceous to of Canada',26:'Zamek Hill assemblage, Cretaceous to of Antarctica',27:'Broken Arrow Bend, Cretaceous of the United States',28:'Takli, Nagpur, Cretaceous of India',29:'773 Hill Gould & Shattuck, Cretaceous of the United States',30:'Palacio de los Loros 1, Paleocene of Argentina',31:'Genesee, Paleocene of Canada',32:'Aspen (2), Cretaceous of the United States',33:'Protection Formation (Nanaimo Group), Cretaceous of Canada',34:'Cabo Shirreff, Isla Livingston (Cretácico Inferior), Cretaceous of Antarctica',35:'Mahurzari, Paleogene of India',36:'Minnekahta Falls, Cretaceous of the United States',37:'Upper Blairmore, Cretaceous of Canada',38:'Comox Coalfield 3769, Cretaceous to of Canada',39:'Well Azx-1, Muglad Basin, Sample Az-04, Cretaceous to of Sudan',40:'Well Khx-1, Muglad Basin, Sample Kh-15, Cretaceous to of Sudan',41:'P3651, Cretaceous of the United States',42:'Reading Beds - Broxbourne, Paleocene of the United Kingdom',43:'L.J. Hickey locality Bugger to get to (Cedar Mtn.), Paleocene of the United States',44:'Puerto Visser, Paleocene of Argentina',45:'D1 - San Ramon section, Paleocene of Argentina',46:'Cerro Cachetamán, Cretaceous of Argentina',47:'Mirador Formation, Norte de Santander, Eocene of Colombia',48:'KJ9403 (DMNH 947), Cretaceous of the United States',49:'Cethana Dam, Oligocene of Australia',50:'Comox Coalfield 3768, Cretaceous to of Canada',51:'Hot Springs 2, Cretaceous of the United States',52:'Alcoa Anglesea locality, Site II , Lens A, Eocene of Australia',53:'Upper Belcourt Ridge, Cretaceous of Canada',54:'The Flora of the Early Miocene Brandon Lignite, Vermont, Cenozoic of USA',55:'Höganäs AB kaolin quarry at the Asen locality in the Kristianstad Basin, Scania, Cretaceous of Sweden'}
    Plants = [[y('Marchantiopsida','Sphaerocarpales','unknown','Aequitriradites cf. acusus'),y('Pinopsida','Pinales','Araucariaceae','Araucariacites hungaricus, Araucariacites australis'),y('Angiospermae','unknown','unknown','Afropollis jardinus, Afropollis cf. operculatus, Stellatopollis barghornii'),y('Angiospermae','Nymphaeales','Nymphaeaceae','Dichastopollenites sp.'),y('Gnetaceaepollenites','unknown','unknown','Gnetaceaepollenites fissuratus'),y('Cycadopsida','Gnetales','unknown','Steevesipollenites sinuosus'),y('Cycadopites','unknown','unknown','Cycadopites sp., Cycadopites carpentieri'),y('Ginkgoopsida','Ephedrales','Ephedraceae','Ephedripites spp., Ephedripites strigatus, Ephedripites jansonii, Ephedripites cf. multicostatus'),y('Ginkgoopsida','Coniferales','Cheirolepidiaceae','Classopollis spp.'),y('Pteridopsida','unknown','unknown','Concavissimisporites cf. variverrucatus, Concavissimisporites punctatus, cf. Echinatisporis varispinosus, Deltoidospora diaphana, Matonisporites sp.'),y('Microfoveolatosporis','unknown','unknown','Microfoveolatosporis spp.'),y('Polypodiopsida','Schizaeales','Polypodiidae','Appendicisporites aegyptiaca, Cicatricosisporites spp.'),y('Polypodiopsida','Salviniales','Polypodiidae','Crybelosporites cf. pannuceus'),y('Trilobosporites','unknown','unknown','Trilobosporites laevigatus'),y('Pilosisporites','unknown','unknown','Pilosisporites trichopapillosus'),y('Verrucosisporites','unknown','unknown','Verrucosisporites obscurilaesuratus, Verrucosisporites rotundus')],[y('Marchantiopsida','Sphaerocarpales','unknown','Aequitriradites verrucosus'),y('Pinopsida','Pinales','Araucariaceae','Balmeisporites sp., Araucariacites sp.'),y('Angiospermae','Austrobaileyales','Trimeniaceae','Cretacaeiporites densimurus'),y('Cycadopites','unknown','unknown','Cycadopites spp.'),y('Cycadopites','Coniferales','unknown','Callialasporites trilobatus, Callialasporites dampieri'),y('Cycadopites','Coniferales','Cheirolepidiaceae','Classopollis sp.'),y('Pteridopsida','unknown','unknown','Concavissimisporites variverrucatus, Osmundacidites wellmanfi, Deltoidospora spp., Ischyosporites spp.'),y('Polypodiopsida','Schizaeales','Polypodiidae','Biretisporites potoniaei, Cicatricosisporites spo'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleicheniidites spp.'),y('Polypodiopsida','Salviniales','Polypodiidae','Crybelosporites sp.'),y('Pilosisporites','unknown','unknown','Pilosisporites sp.'),y('Concavisporites','unknown','unknown','Concavisporites sp.'),y('Pinidae','Cupressales','Pinidae','Inaperturopollenites spp.')],[y('Bryopsida','unknown','unknown','Stereisporites sp., Stereisporites antiquasporites'),y('Magnoliopsida','unknown','unknown','Senipites tercrassata, Psilatricolporites sp., Psilatricolporites sp., Quadraplanus brossus, Retitricolporites sp., Retitricolporites chubutensis, Haloragacidites harrisii, Forcipites longus'),y('Pinopsida','Pinales','Araucariaceae','Araucariacites australis'),y('Angiospermae','Malvales','Bombacaceae','Bombacacidites sp.'),y('Angiospermae','Gunnerales','Gunneraceae','Tricolpites reticulatus'),y('Angiospermae','Asterales','Asteraceae','Tubulifloridites lilliei'),y('Angiospermae','Proteales','unknown','Proteacidites sp. A, Proteacidites angulatus'),y('Angiospermae','Proteales','Proteaceae','Propylipollis sp., Propylipollis ambiguus, Lewalanipollis senectus, Beaupreaidites sp., Triporopollenites minor'),y('Angiospermae','Proteales','unknown','Triatriopollenites sp., Triatriopollenites lateflexus, Peninsulapollis gillii, Peninsulapollis askiniae, Peninsulapollis cf. truswelliae'),y('Angiospermae','Liliales','unknown','Liliacidites regularis, Liliacidites cf. crassibaculatus, Liliacidites variegatus'),y('Angiospermae','Arecales','Arecaceae','Spinizonocolpites hialinus'),y('Angiospermae','Arecales','unknown','Arecipites minutiscabratus, Longapertites aff. vaneendenburgi'),y('Dicotyledoneae','Ericales','Ericaceae','Ericipites microtectanum'),y('Angiospermae','unknown','unknown','Sparganiaceaepollenites sp.'),y('Angiospermae','Piperales','Aristolochiaceae','Rosannia manika'),y('Vitreisporites','unknown','unknown','Vitreisporites pallidus'),y('Psilatricolpites','unknown','unknown','Psilatricolpites inargutus'),y('Lycopodiopsida','Isoetales','Isoetaceae','Echinosporis sp.'),y('Lycopodiopsida','Lycopodiales','Lycopodiaceae','Sestrosporites pseudoalveolatus, Retitriletes austroclavatidites'),y('Lycopodiopsida','Lycopodiales','unknown','Camarozonosporites spp., Camarozonosporites ohaiensis'),y('Lycopodiopsida','Selaginellales','Selaginellaceae','Ceratosporites equalis, Densoisporites velatus'),y('Cycadopites','unknown','unknown','Cycadopites sp.'),y('Cycadopites','Coniferales','Podocarpaceae','Podocarpidites spp., Podocarpidites ellipticus, Podocarpidites elegans, Gamerroites psilasaccus, Trisaccites microsaccatum, Phyllocladidites mawsonii, Lygistepollenites florinii, Microcachryidites antarcticus'),y('Cycadopites','Coniferales','Cheirolepidiaceae','Classopollis sp.'),y('Pteridopsida','unknown','unknown','Grapnelispora evansii, Laevigatosporites ovatus, Trilites sp., Trilites volkheimerii, Baculatisporites comaumensis, Cyathidites australis, Cyathidites patagonicus, Cyathidites minor, Peromonolites sp., Peromonolites sp., Peromonolites vellosus, Leiotriletes sp., Deltoidospora sp.'),y('Microfoveolatosporis','unknown','unknown','Microfoveolatosporis sp.'),y('Polypodiopsida','Cyatheales','Polypodiidae','Cyatheacidites annulatus'),y('Polypodiopsida','Schizaeales','Polypodiidae','Biretisporites sp., Convolutispora sp.'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Clavifera triplex, Gleicheniidites sp., Gleicheniidites senonicus'),y('Polypodiopsida','Salviniales','Polypodiidae','Azollopsis tormentosa'),y('Polypodiopsida','Osmundales','Polypodiidae','Rugulatisporites chubutensis'),y('Lycopsida','unknown','unknown','Foveotriletes cf. parvirectus'),y('Verrucosisporites','unknown','unknown','Verrucosisporites sp')],[y('Magnoliopsida','unknown','unknown','Senipites tercrassata, Clavatricolpites sp., Rousea microreticulata, Rousea patagonica, Psilatricolporites sp., Psilatricolporites cf. salamanquensis, Quadraplanus brossus, Periporopollenites demarcatus, Haloragacidites trioratus, Tricolporites sp., Rhoipites baculatus, Rhoipites minusculus, Forcipites "sp. A", Forcipites sabulosus'),y('Angiospermae','Fagales','unknown','Nothofagidites dorotensis, Nothofagidites kaitangataensis, Nothofagidites nana, Nothofagidites saraensis'),y('Angiospermae','Gunnerales','Gunneraceae','Tricolpites sp., Tricolpites sp., Tricolpites bibaculatus, Tricolpites reticulatus'),y('Angiospermae','Aquifoliales','Aquifoliaceae','Ilexpollenites salamanquensis'),y('Angiospermae','Proteales','unknown','Proteacidites subscabratus, Proteacidites parvus, Proteacidites tenuiexinus, Proteacidites beddoesii'),y('Angiospermae','Proteales','Proteaceae','Beaupreaidites sp., Beaupreaidites elegansiformis, Triporopollenites cf. ambiguus'),y('Angiospermae','Proteales','unknown','Triatriopollenites bertelsii, Triatriopollenites lateflexus, Peninsulapollis sp., Peninsulapollis truswellidae, Peninsulapollis cf. truswellidae, Peninsulapollis gillii, Peninsulapollis askiniae'),y('Angiospermae','Liliales','unknown','Liliacidites sp., Liliacidites kaitangataensis, Liliacidites cf. regularis, Liliacidites vermireticulatus, Liliacidites variegatus, Liliacidites crassilabratus'),y('Angiospermae','Arecales','Arecaceae','Spinizonocolpites hialinus'),y('Angiospermae','Arecales','unknown','Clavamonocolpites sp., Arecipites minutiscabratus, Longapertites patagonicus'),y('Dicotyledoneae','Ericales','Ericaceae','Ericipites scabratus'),y('Angiospermae','unknown','unknown','Sparganiaceaepollenites barungensis'),y('Angiospermae','Canellales','unknown','Pseudowinterapollis couperi'),y('Angiospermae','Psilatricolpites','unknown','Psilatricolpites sp., Psilatricolpites patagonicus')],[y('Angiospermae','Magnoliales','unknown','Endressinia brasiliana')],[y('Angiospermae','Laurales','Calycanthaceae','Virginianthus calycanthoides'),y('Angiospermae','unknown','unknown','Appomatoxia ancistrophora'),y('Angiospermae','Coniferales','Cupressaceae','Athrotaxis cf. ungeri'),y('Angiospermae','Coniferales','Cheirolepidiaceae','Pseudofrenelopsis nathorstiana, Watsoniocladus virginiensis, Watsoniocladus florinii, Watsoniocladus ramonensis, Watsoniocladus valdensis, Watsoniocladus pseudoexpansum, Watsoniocladus obatae, Watsoniocladus sp.')],[y('Bullasporis','unknown','unknown','Bullasporis bifidum'),y('Angiospermae','Chloranthales','Chloranthaceae','Clavatipollenites sp.'),y('Cycadopsida','Gnetales','unknown','Equisetosporites multicostatus'),y('Cycadopsida','Coniferales','Cheirolepidiaceae','Classopollis classoides'),y('Polypodiopsida','Schizaeales','Polypodiidae','Klukisporites scoberis'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleicheniidites feronensis'),y('Polypodiopsida','Gemmatriletes','unknown','Gemmatriletes densegemmotus')],[y('Commelinids','Commelinales','Pontederiaceae','Eichhornia sp.')],[y('Pinopsida','Pinales','Araucariaceae','Brachyphyllum squamosum'),y('Pinopsida','Pinales','Taxaceae','Moriconia cyclotoxon, Geinitzia reichenbachii'),y('Angiospermae','Proteales','Platanaceae','Platanocarpus carolinensis, Platananthus hueberi'),y('Angiospermae','Zingiberales','Zingiberaceae','Spirematospermum chandlerae'),y('Coniferales','Coniferales','unknown','Androvettia statenensis, Androvettia carolinensis')],[y('Angiospermae','Poales','Poaceae','Eragrosites changii'),y('Angiospermae','Poales','Cyperaceae','Liaoxia chenii')],[y('Coniferales','Coniferales','unknown','Callialasporites spp.'),y('Coniferales','Coniferales','Cheirolepidiaceae','Classopollis spp.'),y('Coniferales','Coniferales','Podocarpaceae','Podocarpidites ellipticus, Podocarpidites marwickii, Podocarpidites parviauriculata, Podocarpidites vestitus, Microcachrydites spp., Trichotomosulcites microsaccatus'),y('Cycadopites','unknown','unknown','Cycadopites spp.'),y('Pinopsida','Pinales','Araucariaceae','Cyclusphaera sp., Cyclusphaera psilata, Cyclusphaera radiata, Araucariacites australis'),y('Angiospermae','Chloranthales','Chloranthaceae','Clavatipollenites spp.'),y('Angiospermae','Arecales','Arecaceae','Spinizonocolpites sp.'),y('Angiospermae','Arecales','Arecaceae','Arecipites sp.'),y('Angiospermae','Alismatales','unknown','Pennipollis peroreticulatus'),y('Angiospermae','unknown','unknown','Retimonocolpites spp.'),y('Couperisporites','unknown','unknown','Couperisporites lilianae'),y('Staplinisporites','unknown','unknown','Staplinisporites caminus'),y('Taurocusporites','unknown','unknown','Taurocusporites sp., Taurocusporites segmentatus'),y('Foraminisporis','unknown','unknown','Foraminisporis asymmetricus, Foraminisporis dailyi'),y('Trilobosporites','unknown','unknown','Trilobosporites trioreticulosus'),y('Polypodiopsida','Cyatheales','Polypodiidae','Cibotiidites sp., Cyatheacidites sp., Cyatheacidites tectifera'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Ornamentifera spp., Gleicheniidites sp., Gleicheniidites sp., Gleicheniidites sp., Gleicheniidites senonicus'),y('Polypodiopsida','Schizaeales','Polypodiidae','Ruffordiaspora australiensis, Ruffordiaspora cardielensis, Biretisporites cf. potoniei, Cicatricosisporites sp., Cicatricosisporites sp., Cicatricosisporites cf. hallei, Cicatricosisporites cuneiformis, Cicatricosisporites hughesii, Cicatricosisporites pramparoana, Cicatricosisporites annulatus, Nodosisporites sp., Appendicisporites sp., Appendicisporites potomacensis'),y('Polypodiopsida','Salviniales','Polypodiidae','Crybelosporites striatus'),y('Muricingulisporis','unknown','unknown','Muricingulisporis annulatus'),y('Pteridopsida','unknown','unknown','Tuberculatosporites sp., Concavissimisporites apiverrucatus, Concavissimisporites punctatus, Cyathidites australis, Cyathidites patagonicus, Osmundacidites wellmannii, Granulatisporites spp., Baculatisporites comaumensis, Baculatisporites kachaikensis'),y('Lycopsida','unknown','unknown','Leptolepidites macroverrucosus'),y('Ginkgocycadophytus','unknown','unknown','Ginkgocycadophytus sp.'),y('Brenneripollis','unknown','unknown','Brenneripollis sp.'),y('Lycopodiopsida','Lycopodiales','Lycopodiaceae','Retitriletes austroclavatidites, Retitriletes cf. eminulus, Reticulatisporites arcuatus'),y('Lycopodiopsida','Lycopodiales','unknown','Dictyotosporites complex'),y('Lycopodiopsida','Selaginellales','Selaginellaceae','Densoisporites velatus, Densoisporites corrugatus, Antulsporites baculatus, Foveosporites sp., Foveosporites sp., Foveosporites foveoreticulatus, Neoraistrickia sp., Neoraistrickia truncatus, Ceratosporites equalis'),y('Balmeiopsis','unknown','unknown','Balmeiopsis limbatus'),y('Sotasporites','unknown','unknown','Sotasporites sp.'),y('Sotasporites','unknown','unknown','Sotasporites elegans')],[y('conifers','Pinales','Araucariaceae','Agathis sp.'),y('conifers','Pinales','Podocarpaceae','Dacrycarpus sp.'),y('conifers','Pinales','Podocarpaceae','Phyllocladus sp.'),y('conifers','Pinales','Podocarpaceae','Dacrydium sp.'),y('conifers','Pinales','Podocarpaceae','Halocarpus'),y('Angiospermae','Apiales','Araliaceae','cf. Schefflera'),y('Angiospermae','Fagales','Casuarinaceae','Gymnostoma sp.'),y('Angiospermae','Sapindales','Sapindaceae','cf. Alectryon'),y('Angiospermae','','',''),y('Angiospermae','Fagales','Nothofagaceae','Nothofagus sp.'),y('Angiospermae','Proteales','Proteaceae','unknown'),y('Angiospermae','Ericales','Ericaceae','unknown')],[y('Marchantiopsida','Sphaerocarpales','unknown','Aequitriradites spp.'),y('Bryopsida','unknown','unknown','Triporoletes reticulatus'),y('Pinopsida','Pinales','Araucariaceae','Balmeisporites sp.'),y('Angiospermae','unknown','unknown','Afropollis'),y('Angiospermae','Liliales','unknown','Liliacidites sp.'),y('Angiospermae','unknown','unknown','Retimonocolpites sp.'),y('Cycadopites','unknown','unknown','Cycadopites spp.'),y('Ginkgoopsida','Ephedrales','Ephedraceae','Ephedripites spp., Ephedripites jansonii'),y('Ginkgoopsida','Coniferales','Cheirolepidiaceae','Classopollis spp.'),y('Pteridopsida','unknown','unknown','Concavissimisporites sp., Concavissimisporites punctatus, Cyathidites sp., Deltoidospora spp.'),y('Polypodiopsida','Schizaeales','Polypodiidae','Appendicisporites spp., Biretisporites sp., Biretisporites potoniaei, Cicatricosisporites spp., Cicatricosisporites orbiculatus'),y('Triplanosporites','unknown','unknown','Triplanosporites sp.'),y('Charophyceae','unknown','Zygnemataceae','Ovoidites parvus'),y('Dinophyceae','unknown','unknown','Spiniferites sp.'),y('Dinophyceae','Gonyaulacales','Ceratiaceae','Odontochitina operculata'),y('Dinophyceae','Gonyaulacales','Gonyaulacaceae','Florentinia cf. khaldunii, Florentinia sp., Florentinia laciniata, Oligosphaeridium complex'),y('Dinophyceae','Gonyaulacales','Areoligeraceae','Circulodinium deflandrei')],[y('Angiospermae','Saxifragales','Altingiaceae','Microaltingia apocarpela sp., Microaltingia apocarpela sp.'),y('Angiospermae','Pandanales','Triuridaceae','Nuhliantha nyanzaiana sp., Mabelia archaia sp., Mabelia connatifila sp.'),y('Angiospermae','Magnoliid','unknown','Detrusandra mystagoga, Cronquistiflora sayrevillensis'),y('Dicotyledoneae','Ericales','unknown','Paleoenkianthus sayrevillensis, Paleoenkianthus sayrevillensis'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Boodlepteris turoniana'),y('Chloranthistemon','unknown','unknown','Chloranthistemon crossmanensis n. sp.')],[y('Polypodiopsida','Equisetales','Equisetidae','Equisetum florissantense'),y('Polypodiopsida','Polypodiales','Polypodiidae','Dryopteris guyottii'),y('Carpites','unknown','unknown','Carpites miliodes, Carpites gemmaceus'),y('Musci','unknown','Mniaceae','Archaeomnium brownii'),y('Jungermanniopsis','unknown','unknown','Jungermanniopsis cockerellii'),y('unknown','Coniferales','Cupressaceae','Sequoia affinis, Chamaecyparis linguaefolia, Juniperus haydenii'),y('Ginkgoopsida','Ephedrales','Ephedraceae','Ephedra miocenica'),y('Angiospermae','Najadales','Najadaceae','Najadopsis rugulosa'),y('Angiospermae','Najadales','Vallisneriaceae','Limnobium obliteratum'),y('Angiospermae','Alismatales','Potamogetonaceae','Palaeopotamogeton florissanti, Potamogeton geniculatus, Potamogeton verticillatus'),y('Dicotyledoneae','Ericales','Styracaceae','Halesia reticulata'),y('Dicotyledoneae','Myrtales','Myrtaceae','Eugenia arenaceaeformis'),y('Angiospermae','Solanales','Convolvulaceae','Convolvulites orichitus'),y('Angiospermae','Solanales','Solanaceae','Florissantia speirii'),y('Angiospermae','Ranunculales','Berberidaceae','Mahonia marginata, Mahonia obliqua sp., Mahonia subdenticulata'),y('Angiospermae','Proteales','Proteaceae','Lomatites spinosa'),y('Angiospermae','Proteales','Platanaceae','Platanus florissanti sp.'),y('Angiospermae','Caryophyllales','unknown','Phenanthera petalifera'),y('Angiospermae','Lamiales','Oleaceae','Osmanthus praemissa'),y('Angiospermae','Garryales','Eucommiaceae','Eucommia sp.'),y('Angiospermae','Dipsacales','Adoxaceae','Sambucus newtoni'),y('Angiospermae','Apiales','Araliaceae','Panax andrewsii, Oreopanax dissecta'),y('Angiospermae','Asterales','Asteraceae','Aster florissantia'),y('Angiospermae','Aquifoliales','Aquifoliaceae','Ilex knightiaefolia'),y('Angiospermae','Saxifragales','Grossulariaceae','Ribes errans sp.'),y('Angiospermae','Saxifragales','Saxifragaceae','Hydrangea fraxinifolia, Philadelphus minutus'),y('Angiospermae','Laurales','Lauraceae','Sassafras hesperia, Persea florissantia sp., Lindera coloradica'),y('Angiospermae','Liliales','Smilacaceae','Smilax labidurommae'),y('Angiospermae','Dioscoreales','Dioscoreaceae','Dioscorea sp.'),y('Angiospermae','Arecales','Arecaceae','Palmites sp.'),y('Angiospermae','Poales','Poaceae','Stipa florissanti'),y('Angiospermae','Poales','Typhaceae','Typha lesquereuxi'),y('Angiospermae','Poales','Juncaceae','Juncus crassulus'),y('Angiospermae','Poales','Cyperaceae','Cyperacites lacustris'),y('Angiospermae','Vitales','Vitaceae','Parthenocissus osbornii'),y('Angiospermae','Malvales','Thymelaeaceae','Daphne septentrionalis'),y('Angiospermae','Malvales','Sterculiaceae','Sterculia rigida'),y('Angiospermae','Malvales','Tiliaceae','Tilia populifolia'),y('Angiospermae','Sapindales','Rutaceae','Ptelea cassiodes, Ptelea sp.'),y('Angiospermae','Sapindales','Burseraceae','Bursera serrulata'),y('Angiospermae','Sapindales','Anacardiaceae','Astronium truncatum, Schmaltzia vexans, Cotinus fraterna, Rhus lesquereuxi, Rhus obscura, Rhus stellariaefolia'),y('Angiospermae','Sapindales','Simaroubaceae','Ailanthus americana'),y('Angiospermae','Sapindales','Sapindaceae','Koelreuteria alleni, Acer oregonianum, Acer macginitiei, Acer florissanti, Dipteronia insignis, Dipteronia brownii, Dodonaea umbrina, Cardiospermum terminalis, Sapindus coloradensis'),y('Angiospermae','Sapindales','Meliaceae','Trichilia florissanti, Cedrela lancifolia'),y('Angiospermae','Crossosomatales','Staphyleaceae','Staphylea acuminata'),y('Angiospermae','Celastrales','Celastraceae','Celastrus typica'),y('Angiospermae','Fagales','Juglandaceae','Carya libbeyi, Carya florissantensis'),y('Angiospermae','Fagales','Fagaceae','Castanea dolichophylla, Quercus dumosoides, Quercus orbata, Quercus perdayana sp., Quercus schottii, Quercus lyratiformis, Quercus peritula, Quercus scudderi, Quercus balaninorum, Quercus knowltoniana, Quercus mohavensis, Fagopsis longifolia'),y('Angiospermae','Malpighiales','Euphorbiaceae','Croton furculatum, Euphorbia minuta'),y('Angiospermae','Malpighiales','Salicaceae','Populus pyrifolia, Populus crassa, Populites heeri, Salix coloradica sp., Salix ramaleyi, Salix libbeyi'),y('Angiospermae','Fabales','Fabaceae','Vicia sp., Phaca wilmattae, Cercis parvifolia'),y('Angiospermae','Rosales','Papilionaceae','Leguminosites lespedezoides sp., Phaseolites dedal'),y('Angiospermae','Rosales','Pomaceae','Malus florissantensis, Malus pseudocredneria, Crataegus copeana, Crataegus hendersoni, Crataegus nupta, Crataegus sp., Amelanchier scudderi'),y('Angiospermae','Rosales','Rhamnaceae','Rhamnites pseudostenophyllus, Colubrina spireafollia, Zizyphus florissanti'),y('Angiospermae','Rosales','Mimosaceae','Prosopis linearfolia'),y('Angiospermae','Rosales','unknown','Caesalpinites acuminatus, Caesalpinites coloradicus'),y('Angiospermae','Rosales','Rosaceae','Prunus gracilis, Vauquelinia coloradensis, Cercocarpus myricaefolius, Rosa hilliae, Rubus coloradensis'),y('Angiospermae','Rosales','Ulmaceae','Ulmus tenuinervis, Cedrelospermum sp., Cedrelospermum lineatum1, Celtis mccoshii'),y('Angiospermae','Rosids','Moraceae','Ficus florissantia, Ficus florissantella, Ficus bruesi, Morus symmetrica'),y('Angiospermae','Piperales','Aristolochiaceae','Aristolochia mortua'),y('Pinopsida','Pinales','Taxaceae','Torreya geometrorum'),y('Pinopsida','Pinales','Pinaceae','Pinus wheeleri, Pinus hambachi, Pinus florissanti, Picea lahontense, Picea magna sp., Abies longirostris'),y('Antholithes','unknown','unknown','Antholithes pediloides, Antholithes amoenus'),y('Ascomycota','Sphaeriales','Sphaeriaceae','Didymosphaeria betheli'),y('Hexactinellida','Lychniscosa','Camerospongiidae','Robinia lesquereuxi'),y('Deviacer','unknown','unknown','Deviacer sp.'),y('Chaneya','unknown','unknown','Chaneya tenuis')],[y('Pteridopsida','Filicales','unknown','Cladophlebis dakotensis'),y('Polypodiopsida','Schizaeales','Polypodiidae','Ruffordia cf. goepperti'),y('Polypodiopsida','Polypodiales','Polypodiidae','Asplenium dicksonianum, Onychiopsis psilotoides'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleichenia nordenskioldi'),y('Carpolithus','unknown','unknown','Carpolithus belviderensis'),y('Coniferales','Coniferales','unknown','Abietites ernestinae'),y('Coniferales','Coniferales','Cupressaceae','Sequoia condita, Cupressinoxylon cheyennense'),y('Dicotyledoneae','unknown','unknown','Araliopsoides cretacea'),y('Angiospermae','Apiales','Araliaceae','Araliaceae'),y('Angiospermae','Laurales','Lauraceae','Sassafras mudgli'),y('Angiospermae','Ceratophyllales','Ceratophyllaceae','Ceratophyllum sp.'),y('Angiospermae','Vitales','Vitaceae','Vitiphyllum sp.'),y('Angiospermae','Malvales','Sterculiaceae','Sterculia mucronata'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis belviderensis, Sapindopsis brevifolia'),y('Bennettitales','Bennettitales','unknown','Cycadeospermum lineatum'),y('Bennettitales','Bennettitales','Bennettitaceae','Cycadeoidea munita'),y('Pinopsida','unknown','unknown','Feistmantelia oblonga'),y('Pinopsida','Pinales','Pinaceae','Pinus longifolius, Pinus sp.')],[y('Angiospermae','Ranunculales','unknown','Teixeiraea lusitanica'),y('Angiospermae','Nymphaeales','Nymphaeaceae','Monetianthus mirus')],[y('Angiospermae','Proteales','Platanaceae','Platanocarpus brookensis, Aquia brookensis'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis sp.')],[y('Angiospermae','Trochodendrales','unknown','Tetracentronites panochetris'),y('Pinopsida','Pinales','Taxodiaceae','Margeriella cretacea')],[y('Scabramonocolpites','unknown','unknown','Scabramonocolpites sp.'),y('Pteridopsida','unknown','unknown','Cyathidites sp., Osmundacidites cf. primarius'),y('Polypodiopsida','Schizaeales','Polypodiidae','Cicatricosisporites mohroides, Cicatricosisporites sp., Cicatricosisporites hallei, Klukisporites sp.'),y('Triporites','unknown','unknown','Triporites sp.'),y('Psilatricolpites','unknown','unknown','Psilatricolpites sp., Psilatricolpites spp.'),y('Retitricolpites','unknown','unknown','Retitricolpites sp., Retitricolpites sp., Retitricolpites sp., Retitricolpites sp., Retitricolpites sp., Retitricolpites operculatus'),y('Striatricolpites','unknown','unknown','Striatricolpites sp.'),y('Coniferales','Coniferales','Cheirolepidiaceae','Classopollis greaterthanthirty, Classopollis lessthanthirty, Classopollis spinosus'),y('Ginkgoopsida','Ephedrales','Ephedraceae','Ephedripites jansonii, Ephedripites barghorrrnii, Ephedripites brasiliensis, Ephedripites sp.'),y('Cycadopsida','Cycadales','unknown','Eucommiidites sp.'),y('Cycadopsida','Gnetales','unknown','Elaterosporites klaszi, Elaterosporites verrucatus, Elaterosporites protensus, Elaterocolpites castelani, Steevesipollenites binodosus, Steevesipollenites multilineatus'),y('Lycopodiopsida','Lycopodiales','Lycopodiaceae','Lycopodiumsporites sp.'),y('Galeacornea','unknown','unknown','Galeacornea causea'),y('Angiospermae','unknown','unknown','Monosulcites sp., Afropollis jardinus, Retimonocolpites sp.'),y('Dicotyledoneae','unknown','unknown','Triorites sp., Triorites africaensis, Stephanocolpites sp.'),y('Angiospermae','Buxales','Didymelaceae','Hexaporotricolpites lamellaferus'),y('Angiospermae','Austrobaileyales','Trimeniaceae','Cretacaeiporites scabratus, Cretacaeiporites mulleri'),y('Verrutriletes','unknown','unknown','Verrutriletes sp.'),y('Retitriporites','unknown','unknown','Retitriporites sp.'),y('Reyrea','unknown','unknown','Reyrea polymorphus'),y('Senegalosporites','unknown','unknown','Senegalosporites petrobrasi'),y('Chomotriletes','unknown','unknown','Chomotriletes sp.'),y('Perotriletes','unknown','unknown','Perotriletes pannuceus'),y('Elateropollenites','unknown','unknown','Elateropollenites jardinei')],[y('Pteridopsida','unknown','unknown','Deltoidospora australis'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleicheniidites senonicus'),y('Foraminisporis','unknown','unknown','Foraminisporis sp.'),y('Angiospermae','Gunnerales','Gunneraceae','Tricolpites sp.'),y('Pinopsida','Pinales','Araucariaceae','Araucariacites fissus')],[y('Reptilia','Saurischia','unknown','cf. Irenesauripus sp.'),y('Cupuliferoidaepollenites','unknown','unknown','Cupuliferoidaepollenites sp.'),y('Pinopsida','Pinales','Pinaceae','Pinuspollenites sp.'),y('Angiospermae','Fagales','Myricaceae','Myricipites sp.'),y('Abietineaepollenites','unknown','unknown','Abietineaepollenites sp.'),y('Cedripites','unknown','unknown','Cedripites sp.'),y('Bryopsida','unknown','unknown','Triporoletes sp.'),y('Coniferales','unknown','unknown','Taxodiaceaepollenites sp.'),y('Polypodiopsida','Polypodiales','Polypodiidae','Polypodiisporites sp.'),y('Pteridopsida','unknown','unknown','Leiotriletes sp., Deltoidospora sp.'),y('Quercoidites','unknown','unknown','Quercoidites sp.')],[y('Carpolithus','unknown','unknown','Carpolithus subfusiformis, Carpolithus gracilis, Carpolithus stonei, Carpolithus thunbergioides'),y('Neuroraphe','unknown','unknown','Neuroraphe obovatum'),y('Tricarpellites','unknown','unknown','Tricarpellites communis'),y('Angiospermae','unknown','unknown','Leyrida bilocularis, Lagenella alata'),y('Dicotyledoneae','Ericales','Epacridaceae','Leucopogon quadrilocularis'),y('Dicotyledoneae','Myrtales','Lythraceae','Tamesicarpum polyspermum'),y('Dicotyledoneae','Myrtales','Myrtaceae','Aldwickia venablesi'),y('Angiospermae','Violales','Flacourtiaceae','Oncoba variabilis'),y('Angiospermae','Violales','unknown','Exechinocarpus'),y('Angiospermae','Ranunculales','Menispermaceae','Tinospora rugosa, Tinospora wilkinsoni sp., Tinospora excavata'),y('Angiospermae','Proteales','Sabiaceae','Meliosma jenkinsii, Meliosma cantiensis'),y('Angiospermae','Cornales','Cornaceae','Lanfrancia subglobosa, Langtonia bisulcata, Cornus ettingshausenii, Cornus multilocularis, Portnallia sheppeyensis'),y('Angiospermae','Cornales','Alangiaceae','Alangium jenkinsi'),y('Angiospermae','Cornales','Nyssaceae','Nyssa cooperi, Nyssa sp., Palaeonyssa multilocularis, Protonyssa bilocularis'),y('Angiospermae','Gentianales','unknown','Jenkinsella apocynoides'),y('Angiospermae','Icacinales','Icacinaceae','Iodes multireticulata, Icacinicarya jenkinsi, Icacinicarya foveolata, Icacinicarya reticulata sp., Icacinicarya amygdaloidea sp., Icacinicarya nodulifera, Icacinicarya ovoidea, Icacinicarya platycarpa, Icacinicarya minima, Palaeophytocrene foveolata'),y('Angiospermae','Santalales','Olacaceae','Erythropalum jenkinsi'),y('Angiospermae','Saxifragales','Saxifragaceae','Saxifragispermum spinosissimum'),y('Angiospermae','Dilleniales','Dilleniaceae','Tetracera eocenica, Tetracera crofti sp.'),y('Angiospermae','Laurales','Lauraceae','Cinnamomum ovoideum sp., Cinnamomum oblongum sp., Cinnamomum globulare, Cinnamomum grande, Laurocalyx globularis, Laurocarpum sp., Laurocarpum spp., Laurocarpum cupuliferum sp., Laurocarpum ovoideum, Laurocarpum proteum, Laurocarpum minimum, Laurocarpum minutissimum, Laurocarpum sheppeyense, Laurocarpum inornatum sp., Beilschmiedia oviformis, Beilschmiedia pyriformis, Protoravensara sheppeyensis, Litsea pyriformis'),y('Angiospermae','Arecales','Arecaceae','Nypa burtini, Palmospermum excavatum, Palmospermum sp., Palmospermum jenkinsi, Palmospermum minutum sp., Palmospermum cooperi sp., Palmospermum ornatum sp., Sabal sp.'),y('Angiospermae','Poales','Cyperaceae','Polycarpella caespitosa'),y('Angiospermae','Vitales','Vitaceae','Palaeovitis paradoxa, Vitis subglobosa'),y('Angiospermae','Malvales','Sterculiaceae','Sphinxia ovalis'),y('Angiospermae','Malvales','Malvaceae','Hightea turgida, Hightea elliptica'),y('Angiospermae','Malvales','Tiliaceae','Cantitilia polysperma'),y('Angiospermae','Sapindales','Rutaceae','Canticarya ventricosa, Canticarya sheppeyensis, Citrispermum sheppeyense sp., Shrubsolea jenkinsi'),y('Angiospermae','Sapindales','Anacardiaceae','Lobaticarpum variabile, Pseudosclerocarya lentiformis, Dracontomelon subglobosum, Lannea'),y('Angiospermae','Sapindales','Sapindaceae','Sapindospermum sp., Sapindospermum ovoideum, Sapindospermum revolutum sp., Sapindospermum cooperi sp., Sapindospermum jenkinsi'),y('Angiospermae','Cucurbitales','Cucurbitaceae','Cucurbitospermum cooperi'),y('Angiospermae','Fagales','Juglandaceae','Platycarya richardsoni, Juglandicarya minuta, Juglandicarya cooperi, Juglandicarya depressa, Juglandicarya sp.'),y('Angiospermae','Malpighiales','Euphorbiaceae','Euphorbiotheca minima sp., Euphorbiospermum eocenicum, Euphorbiospermum cooperi sp., Wetherellia variabilis'),y('Angiospermae','Malpighiales','Rhizophoraceae','Palaeobruguiera lata sp., Palaeobruguiera elongata sp.'),y('Angiospermae','Fabales','Fabaceae','Faboidea crassicutis'),y('Angiospermae','Magnoliales','Magnoliaceae','Magnolia subcircularis, Magnolia crassa, Magnolia rugosa sp., Magnolia angusta, Magnolia oblonga sp.'),y('Angiospermae','unknown','unknown','Lagenoidea bilocularis'),y('Magnoliopsida','unknown','unknown','Cupanoides grandis, Cupanoides tumidus'),y('Mastixia','unknown','Mastixiaceae','Mastixia parva, Mastixia cantiensis'),y('Spondias','unknown','unknown','Spondias sheppeyensis')],[y('Angiospermae','Rosales','Rhamnaceae','Eophylica priscastellata')],[y('Pteridopsida','Filicales','unknown','Cladophlebis alberta, Cladophlebis cf. porsildi, Cladophlebis cf. virginiensis, Cladophlebis parva'),y('Gnetopsida','unknown','unknown','Pityophyllum nordenskioeldi'),y('Hepaticae','unknown','unknown','Thallites blairmorensis'),y('Coniferales','Coniferales','Podocarpaceae','Elatocladus acifolia, Podozamites sp.'),y('Coniferales','unknown','unknown','Athrotaxites sp.'),y('Ginkgoopsida','Caytoniales','Caytoniaceae','Sagenopteris cf. mclearni, Sagenopteris williamsii'),y('Ginkgoopsida','Ginkgoales','Ginkgoaceae','Ginkgo aff. digitata, Ginkgo cf. pluripartita'),y('Ctenopteris','unknown','unknown','Ctenopteris sp.'),y('Cycadopsida','Cycadales','Nilssoniaceae','Pseudoctenis aff. hazeltonensis'),y('Cycadopsida','Cycadales','unknown','Nilsonia'),y('Sphenopteridae','Sphenopteridae','unknown','Sphenopteris sp., Sphenopteris cf. latiloba'),y('Angiospermae','Apiales','Araliaceae','Aralia wellingtoniana'),y('Angiospermae','Laurales','Lauraceae','Sassafras parvofilium'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis cf. magnifolia'),y('Angiospermae','Malpighiales','Salicaceae','Populites cf. dawsoni'),y('Angiospermae','Fabales','Fabaceae','Fontainea cf. grandiflora'),y('Angiospermae','Rosids','Moraceae','Ficus ovatifolia'),y('Angiospermae','Bennettitales','unknown','Ptilophyllum cf. columbianum, Ptilophyllum aff. montanense, Pterophyllum aff. plicatum, Pterophyllum cf. rectangulare, Pterophyllum sp., Pseudocycas cf. sp., Pseudocycas cf. unjiga'),y('Pinopsida','Pinales','Taxaceae','Elatides cf. splendida, Elatides cf. curvifolia, Cyparissidium gracile, Protophyllocladus subintegrifolius')],[y('Pteridopsida','Filicales','unknown','Cladophlebis columbiana'),y('Polypodiopsida','Polypodiales','Polypodiidae','Onoclea hebridica, Dryopteris kennerlyi'),y('Coniferales','Coniferales','Cupressaceae','Metasequoia cuneata'),y('Angiospermae','Typhales','Commelinidae','Sparganium sp.'),y('Dicotyledoneae','Parietales','Ternstroemiaceae','Ternstroemites harwoodensis'),y('Angiospermae','Ranunculales','Menispermaceae','Menispermum dauricumoides'),y('Angiospermae','Proteales','Platanaceae','Platanus affinis'),y('Angiospermae','Sapindales','Sapindaceae','Sapindus pacificus'),y('Angiospermae','Crossosomatales','Staphyleaceae','Staphylea usheri'),y('Angiospermae','Celastrales','Celastraceae','Celastrinites wardii'),y('Angiospermae','Fagales','Fagaceae','Dryophyllum ripleyensis'),y('Angiospermae','Malpighiales','Salicaceae','Populus sp.'),y('Angiospermae','Rosales','Rhamnaceae','Rhamnites eminens, Zizyphus areolatus'),y('Angiospermae','Rosales','Rosaceae','Pyrus sp.')],[y('Pteridopsida','Filicales','unknown','Filicales sp., Cladophlebis sp.'),y('Coniferales','Coniferales','Podocarpaceae','Podocarpus sp.'),y('Phyllites','unknown','unknown','Phyllites sp.'),y('Dicotyledoneae','Myrtales','Myrtaceae','Myrciophyllum santacruzensis'),y('Angiospermae','Apiales','Araliaceae','Araliaephyllum aff. quinquelobatus'),y('Angiospermae','Laurales','Monimiaceae','Monimiophyllum aff. antarcticum'),y('Angiospermae','Laurales','Lauraceae','aff. Mespilodaphne longifolia'),y('Angiospermae','Fagales','Nothofagaceae','aff. Nothofagoxylon sp., Nothofagus aff. ulmifolia, Nothofagus cretacea, Nothofagus glaucifolia, Nothofagus zastawniakiae'),y('Angiospermae','Fagales','Betulaceae','aff. Alnus muelleri'),y('Angiospermae','Oxalidales','Elaeocarpaceae','aff. Elaeocarpus sp.'),y('Pinopsida','Pinales','Araucariaceae','Araucaria sp.'),y('Magnoliopsida','unknown','unknown','Dicotylophyllum sp., Dicotylophyllum elegans'),y('Pentaneurum','unknown','unknown','Pentaneurum dusenii'),y('Magnoliidaephyllum','unknown','unknown','Magnoliidaephyllum birkenmajeri'),y('Sterculiaephyllum','unknown','unknown','Sterculiaephyllum australis')],[y('Angiospermae','Malpighiales','Salicaceae','Salix flexuosa, Salix eutawensis')],[y('Angiospermae','unknown','unknown','Viracarpon hexaspermum, Viracarpon elongatum sp., Xulinosprionites sp.'),y('Angiospermae','Apiales','unknown','cf. Hydrocotyle sp.'),y('Angiospermae','Arecales','Arecaceae','Palmoxylon takliense sp., Nipadites sp., Palmocarpon takliensis'),y('Angiospermae','Sapindales','Rutaceae','Paraferonioxylon takliensis'),y('Angiospermae','Zygophyllales','Zygophyllaceae','Guaiacum takliensis'),y('Angiospermae','Fabales','Fabaceae','cf. Faboidea sp., cf. Cassia sp.'),y('Pinopsida','Pinales','Taxaceae','Takliostrobus alatus'),y('Pinopsida','Pinales','Pinaceae','Pityostrobus crassitesta'),y('Reptilia','Saurischia','unknown','Massospondylus rawesi'),y('Actinopteri','Scombriformes','unknown','cf. Sphyraenodus sp.'),y('Charophyceae','Charales','Characeae','Chara malcolmsonii')],[y('Coniferales','Coniferales','unknown','Abietites longifolius'),y('Coniferales','Coniferales','Cupressaceae','Sequoia condita'),y('Angiospermae','Malvales','Sterculiaceae','Sterculia towneri'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis magnifolia')],[y('Angiospermae','unknown','unknown','Dicotyledonae'),y('Angiospermae','Violales','Flacourtiaceae','Banaraphyllum ovatum'),y('Angiospermae','Ranunculales','Menispermaceae','Menispermaceae SA 060, Wilkinsoniphyllum menispermoides'),y('Angiospermae','Laurales','Lauraceae','Laurophyllum piatnitzkyi, Laurophyllum chubutensis'),y('Angiospermae','Vitales','Vitaceae','Cissites patagonicum'),y('Angiospermae','Brassicales','Akaniaceae','Akania sp.'),y('Angiospermae','Malvales','Sterculiaceae','Sterculia acuminataloba'),y('Angiospermae','Malvales','Malvaceae','Malvaceae SA 058'),y('Angiospermae','Sapindales','Sapindaceae','Sapindaceae SA 052'),y('Angiospermae','Fagales','Myricaceae','Myrica premira'),y('Angiospermae','Fagales','Fagaceae','Fagophyllum duseni, Dryophyllum australis'),y('Angiospermae','Fabales','Fabaceae','Fabaceae SA 045'),y('Angiospermae','Rosales','Rosaceae','Rosaceae SA 008'),y('Angiospermae','Rosales','Urticaceae','Urticaceae SA 051'),y('Angiospermae','Nymphaeales','Nymphaeaceae','Paranymphaea aristolochiaformis')],[y('Pteridopsida','Ophioglossales','Ophioglossidae','Botrychium sp.'),y('Polypodiopsida','Polypodiales','Polypodiidae','Woodwardia arctica'),y('Polypodiopsida','Salviniales','Polypodiidae','Azolla schopfii'),y('Coniferales','Coniferales','Cupressaceae','Taxodium dubium, Glyptostrobus nordenskioldi, Metasequoia occidentalis, Fokienia catenulata'),y('Angiospermae','Proteales','Platanaceae','Platanus raynoldsii'),y('Angiospermae','Trochodendrales','Trochodendraceae','Nordenskioldia borealis'),y('Angiospermae','Saxifragales','Cercidiphyllaceae','Cercidiphyllum genesevianum sp., Cercidiphyllum flexuosum, Cercidiphyllum cuneatum, Cercidiphyllum sp.'),y('Angiospermae','Vitales','Vitaceae','Ampelopsis acerifolia, Vitis sp.'),y('Angiospermae','Huerteales','Tapisciaceae','Tapiscia serrata')],[y('Angiospermae','Typhales','Commelinidae','Sparganium aspensis'),y('Angiospermae','Proteales','Proteaceae','Dryandroides lanceolata'),y('Angiospermae','Proteales','Nelumbonaceae','Nelumbo weymouthii'),y('Angiospermae','Laurales','Lauraceae','Laurus aspensis, Sassafras bradleyi'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis schultzii'),y('Angiospermae','Fagales','Fagaceae','Dryophyllum lanceolatum'),y('Angiospermae','Rosales','Rosaceae','Prunus aspensis')],[y('Angiospermae','Proteales','Platanaceae','Platanus affinis'),y('Angiospermae','Ranunculales','Menispermaceae','Menispermum dauricumoides'),y('Dicotyledoneae','Parietales','Ternstroemiaceae','Ternstroemites harwoodensis'),y('Angiospermae','Typhales','Commelinidae','Sparganium sp.'),y('Angiospermae','Sapindales','Sapindaceae','Sapindus pacificus'),y('Angiospermae','Crossosomatales','Staphyleaceae','Staphylea usheri'),y('Angiospermae','Rosales','Rosaceae','Pyrus sp.'),y('Angiospermae','Rosales','Rhamnaceae','Zizyphus areolatus, Rhamnites eminens'),y('Angiospermae','Fagales','Fagaceae','Dryophyllum ripleyensis'),y('Angiospermae','Celastrales','Celastraceae','Celastrinites wardii'),y('Angiospermae','Malpighiales','Salicaceae','Populus sp.'),y('Coniferales','Coniferales','Cupressaceae','Metasequoia cuneata'),y('Polypodiopsida','Polypodiales','Polypodiidae','Dryopteris kennerlyi, Onoclea hebridica'),y('Pteridopsida','Filicales','unknown','Cladophlebis columbiana')],[y('Pteridopsida','unknown','unknown','Trilites tuberculiformis, Cyathidites minor, Cyathidites australis, Laevigatosporites ovatus, Deltoidospora sp.'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Clavifera triplex, Gleicheniidites concavisporites, Gleicheniidites senonicus'),y('Psilatricolpites','unknown','unknown','Psilatricolpites sp.'),y('Retitricolpites','unknown','unknown','Retitricolpites sp.'),y('Coniferales','Coniferales','Podocarpaceae','Podocarpidites otagoensis, Podocarpidites marwickii'),y('Dicotyledoneae','Myrtales','unknown','Myrtaceidites eugeniioides'),y('Angiospermae','Gunnerales','Gunneraceae','Tricolpites baculado'),y('Monoporisporites','unknown','unknown','Monoporisporites sp.'),y('Multicellaesporites','unknown','unknown','Multicellaesporites sp.')],[y('Leeoxylon','unknown','Ampelidaceae','Leeoxylon multiseriatum'),y('Angiospermae','unknown','unknown','Viracarpon sahnii'),y('Dicotyledoneae','unknown','unknown','Chitaloxylon deccanii'),y('Dicotyledoneae','Ericales','Lecythidaceae','Foetidioxylon decanii sp., Barringtonioxylon deccanense sp., Barringtonioxylon eopterocarpum sp.'),y('Dicotyledoneae','Ericales','Ebenaceae','Ebenoxylon mahurzarii'),y('Angiospermae','Santalales','Olacaceae','Ximenioxylon mahurzari'),y('Angiospermae','Arecales','Arecaceae','Palmoxylon mahurzari sp., Palmoxylon eocenum'),y('Angiospermae','Geraniales','unknown','Simarubinium mahurzari'),y('Angiospermae','Malvales','Sterculiaceae','Sterculioxylon baradense'),y('Angiospermae','Malvales','Malvaceae','Hibiscoxylon intertrappeum'),y('Angiospermae','Malvales','Dipterocarpaceae','Vaterioxylon deccanii sp., Shoreoxylon mahurzarii sp.'),y('Angiospermae','Malvales','Tiliaceae','Heliocarpoxylon mahurzarii, Grewioxylon mahurzariense, Grewioxylon intertrappea, Grewioxylon indicum'),y('Angiospermae','Sapindales','Rutaceae','Rutaceoxylon mahurzarii'),y('Angiospermae','Sapindales','Burseraceae','Canarioxylon deccanii'),y('Angiospermae','Sapindales','Anacardiaceae','Anacardioxylon mahurzari'),y('Angiospermae','Sapindales','Simaroubaceae','Ailanthoxylon indicum'),y('Angiospermae','Celastrales','Celastraceae','Perrottetioxylon mahurzari'),y('Angiospermae','Malpighiales','Malpighiaceae','Thryallioxylon deccanii'),y('Angiospermae','Fabales','Fabaceae','Aeschynomene tertiara'),y('Angiospermae','Oxalidales','Elaeocarpaceae','Elaeocarpoxylon antiquum'),y('Magnoliopsida','unknown','unknown','Dryoxylon mahurzarii sp., Dryoxylon intertrappea sp., Dryoxylon tiliaceoides sp.')],[y('Polypodiopsida','Polypodiales','Polypodiidae','Asplenium Dicksonianum'),y('Dicotyledoneae','unknown','unknown','Araliopsoides cretacea'),y('Angiospermae','Vitales','Vitaceae','Cissites ingens')],[y('Pteridopsida','Filicales','unknown','Cladophlebis alberta'),y('Coniferales','Coniferales','Cupressaceae','Sequoia condita'),y('Sphenopteridae','unknown','Sphenopteridae','Sphenopteris erecta'),y('Cinnamomoides','unknown','unknown','Cinnamomoides ovalis'),y('Angiospermae','Apiales','Araliaceae','Araliaephyllum westoni'),y('Angiospermae','Saxifragales','Cercidiphyllaceae','Trochodendroides potomacensis'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis angusta, Sapindopsis belviderensis'),y('Angiospermae','Malpighiales','Salicaceae','Populites dawsoni'),y('Angiospermae','Fabales','Fabaceae','Fontainea grandiflora'),y('Angiospermae','Rosids','Moraceae','Ficus ovatifolia'),y('Angiospermae','Bennettitales','unknown','Pseudocycas cf. unjiga, Zamites tenuinervis'),y('Magnoliopsida','unknown','unknown','Celastrophyllum acutidens')],[y('Coniferales','Coniferales','Cupressaceae','Glyptostrobus comoxensis'),y('Angiospermae','Ranunculales','Menispermaceae','Menispermites acutilobus'),y('Angiospermae','Dilleniales','Dilleniaceae','Dillenites paucidentatus'),y('Angiospermae','Rosales','Rhamnaceae','Rhamnites eminens, Zizyphus cretaceus'),y('Pinopsida','Pinales','Taxaceae','Protophyllocladus polymorphia')],[y('Polypodiopsida','Schizaeales','Polypodiidae','Cicatricosisporites spp.'),y('Polypodiopsida','Salviniales','Polypodiidae','Crybelosporites cf. pannuceus'),y('Monocolpopollenites','unknown','unknown','Monocolpopollenites sp.'),y('Marchantiopsida','Sphaerocarpales','unknown','Aequitriradites cf. acusus'),y('Coniferales','Coniferales','Cheirolepidiaceae','Classopollis spp.'),y('Ginkgoopsida','Ephedrales','Ephedraceae','Ephedripites spp., Ephedripites jansonii, Ephedripites cf. multicostatus'),y('Angiospermae','Proteales','Proteaceae','Echitriporites irregularis'),y('Angiospermae','Santalales','Loranthaceae','Syncolporites marginatus'),y('Pinopsida','Pinales','Araucariaceae','Araucariacites australis'),y('Magnoliopsida','unknown','unknown','Striatopollis sp., Gemmamonocolpites sp., Periretisyncolpites giganteus'),y('Dinophyceae','Peridiniales','Peridiniaceae','Holmwoodinium notatum'),y('Pinidae','Cupressales','Pinidae','Sphaeripollenites')],[y('Pteridopsida','unknown','unknown','Matonisporites sp.'),y('Polypodiopsida','Schizaeales','Polypodiidae','Appendicisporites aegyptiaca, Cicatricosisporites spp.'),y('Pilosisporites','unknown','unknown','Pilosisporites trichopapillosus'),y('Monocolpopollenites','unknown','unknown','Monocolpopollenites sp.'),y('Ginkgoopsida','Ephedrales','Ephedraceae','Ephedripites spp., Ephedripites strigatus, Ephedripites cf. multicostatus'),y('Cycadopsida','Gnetales','unknown','Steevesipollenites sinuosus'),y('Gnetaceaepollenites','unknown','unknown','Gnetaceaepollenites spp.'),y('Angiospermae','unknown','unknown','Afropollis zonatus, Afropollis jardinus'),y('Angiospermae','Caryophyllales','Droseraceae','Droseridites senonicus'),y('Pinopsida','Pinales','Araucariaceae','Araucariacites australis'),y('Pinidae','Cupressales','Pinidae','Inaperturopollenites spp.')],[y('Carpites','unknown','unknown','Carpites lancensis'),y('Coniferales','Coniferales','Cupressaceae','Sequoia dakotensis'),y('Angiospermae','Alismatales','Araceae','Porosia verrucosa'),y('Angiospermae','Cornales','Cornaceae','Cornophyllum wardii'),y('Angiospermae','Lamiales','Oleaceae','Fraxinus leii'),y('Angiospermae','Apiales','Araliaceae','Araliaephyllum artocarpoides'),y('Angiospermae','Dilleniales','Dilleniaceae','Dillenites cleburni'),y('Angiospermae','Poales','Typhaceae','Typha sp.'),y('Angiospermae','Sapindales','Anacardiaceae','Pistacia eriensis'),y('Angiospermae','Fagales','Fagaceae','Dryophyllum subfalcatum')],[y('Carpolithus','unknown','unknown','Carpolithus sp.'),y('Angiospermae','Icacinales','Icacinaceae','Natsiatum eocenicum')],[y('Coniferales','Coniferales','Cupressaceae','Glyptostrobus europaeus'),y('Angiospermae','unknown','unknown','Monocotyledoneae'),y('Angiospermae','Alismatales','Alismataceae','aff. Alismataceae sp.'),y('Angiospermae','Proteales','Platanaceae','Platanus raynoldsii'),y('Angiospermae','Garryales','Eucommiaceae','Eucommia serrata'),y('Angiospermae','Saxifragales','Cercidiphyllaceae','Cercidiphyllum genetrix'),y('Angiospermae','Fagales','Juglandaceae','Hicoria antiquora'),y('Angiospermae','Malpighiales','Salicaceae','Populus" nebraskana')],[y('Angiospermae','Boraginales','Boraginaceae','Cordioxylon prototrichotoma')],[y('Pteridopsida','unknown','unknown','Trilites tuberculiformis, Cyathidites minor, Cyathidites australis, Cyathidites patagonicus, Laevigatosporites ovatus, Leiotriletes sp., Deltoidospora sp.'),y('Polypodiopsida','Cyatheales','Polypodiidae','Cyatheacidites annulatus'),y('Polypodiopsida','Osmundales','Polypodiidae','Rugulatisporites chubutensis'),y('Polypodiopsida','Schizaeales','Polypodiidae','Convolutispora sp'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Clavifera triplex, Gleicheniidites sp., Gleicheniidites senonicus, Gleicheniidites circinidites'),y('Verrucosisporites','unknown','unknown','Verrucosisporites sp.'),y('Lycopsida','unknown','unknown','Foveotriletes cf. parvirectus'),y('Psilatricolpites','unknown','unknown','Psilatricolpites inargutus'),y('Bryopsida','unknown','unknown','Stereisporites sp.'),y('Coniferales','Coniferales','Cheirolepidiaceae','Classopollis sp.'),y('Coniferales','Coniferales','Podocarpaceae','Podocarpidites spp., Podocarpidites ellipticus, Podocarpidites elegans, Microcachryidites antarcticus, Phyllocladidites mawsonii'),y('Lycopodiopsida','Lycopodiales','unknown','Camarozonosporites spp.'),y('Lycopodiopsida','Lycopodiales','Lycopodiaceae','Retitriletes austroclavatidites'),y('Angiospermae','Proteales','unknown','Peninsulapollis cf. truswelliae'),y('Angiospermae','Proteales','Proteaceae','Triporopollenites minor, Echitriporites sp., Lewalanipollis senectus'),y('Angiospermae','Proteales','unknown','Triatriopollenites sp., Triatriopollenites lateflexus, Proteacidites sp.'),y('Angiospermae','Gentianales','Apocynaceae','Cricotriporites sp.'),y('Angiospermae','Gunnerales','Gunneraceae','Tricolpites reticulatus'),y('Angiospermae','Arecales','unknown','Arecipites minutiscabratus'),y('Angiospermae','Fagales','unknown','Nothofagidites dorotensis'),y('Magnoliopsida','unknown','unknown','Psilatricolporites sp., Retitricolporites sp., Haloragacidites harrisii, Forcipites cf. longus, Senipites tercrassata, Gemmatricolpites sp.')],[y('Polypodiopsida','Polypodiales','Polypodiidae','Dryopterites problematicus'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleichenites sp.'),y('Cycadopsida','Cycadales','unknown','Nilsonia'),y('Angiospermae','Lamiales','Bignoniaceae','Bignonites chalianus'),y('Angiospermae','Apiales','Araliaceae','Araliaephyllum cachetamanense'),y('Angiospermae','Laurales','Lauraceae','Laurophyllum proteaefolium, Laurophyllum kurtzi, Laurophyllum hickenii'),y('Angiospermae','Vitales','Vitaceae','Cissites parvifolius'),y('Angiospermae','Malvales','Sterculiaceae','Sterculia platanoides, Sterculia patagonica')],[y('','','','')],[y('Polypodiopsida','Equisetales','Equisetidae','Equisetum UD006'),y('Polypodiopsida','Polypodiales','Polypodiidae','Astralopteris coloradica'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleichenia comptoniaefolia, Gleichenia delicatula, Matonidium brownii'),y('Angiospermae','unknown','unknown','Dicotyledoneae'),y('Angiospermae','Aquifoliales','Aquifoliaceae','Ilex serrata')],[y('Pteridopsida','unknown','unknown','Trilites tuberculiformis'),y('Pteridopsida','Psilotales','Ophioglossidae','Tmesipteris tasmanica'),y('Pteridopsida','unknown','unknown','Cyathidites minor, Cyathidites australis, Cyathidites paleospora, Laevigatosporites ovatus, Peromonolites vellosus, Matonisporites ornamentalis, cf. Ischyosporites irregularis'),y('Polypodiopsida','Cyatheales','Polypodiidae','Cyatheacidites annulatus'),y('Polypodiopsida','Schizaeales','Polypodiidae','Lygodium sp., Lygodium dinmorphyllum, Schizaea sp.'),y('Polypodiopsida','Polypodiales','Polypodiidae','Polypodiidites sp., Verrucatosporites attinatus'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleicheniidites circiniidites, Gleichenia sp., Dictyophyllidites arcuatus'),y('Verrucosisporites','unknown','unknown','Verrucosisporites aff. cristatus'),y('Lycopsida','unknown','unknown','Foveotriletes balteus, Foveotriletes palaequetrus'),y('Granodiporites','unknown','unknown','Granodiporites nebulosus'),y('Bryopsida','unknown','unknown','Stereisporites sp.'),y('Coniferales','Coniferales','Podocarpaceae','Podosporites sensu lato parvisaccites, Podosporites microsaccatus, Lygistepollenites florinii, Phyllocladus sp., Phyllocladus sp., Dacrycarpites australiensis, Lagarostrobos sp., Dacrycarpus mucronatus, Acmopyle glabra, Podocarpidites ostentatus, Podocarpidites spp., Microcachryidites antarcticus, Dacrydium sp., Phyllocladidites mawsonii, Podocarpus sp.'),y('Coniferales','Coniferales','Cupressaceae','Libocedrus morrisonii, Papuacedrus australis'),y('Cycadopites','unknown','unknown','Cycadopites sp.'),y('Cycadopsida','Cycadales','Zamiaceae','Macrozamia australis, Pterostoma aff. anastomosans'),y('Lycopodiopsida','Lycopodiales','Lycopodiaceae','Latrobosporites crassus, Lycopodiumsporites sp., Retitriletes austroclavatidites'),y('Lycopodiopsida','Selaginellales','Selaginellaceae','Herkosporites elliotii'),y('Dicotyledoneae','Ericales','Ericaceae','Ericipites crassiexinus, Ericipites scabratus'),y('Dicotyledoneae','Myrtales','unknown','Myrtaceidites mesonesus, Myrtaceidites parvus'),y('Angiospermae','Proteales','Proteaceae','Proteaceae incertae sedis, Telopea truncata, Euproteaciphyllum rugulatum, Euproteaciphyllum lomatioides, Euproteaciphyllum attenuatum, Euproteaciphyllum ornamentalis, Euproteaciphyllum gevuininoides, Euproteaciphyllum linearis, Euproteaciphyllum cethanicum, Euproteaciphyllum integrifolium, Euproteaciphyllum integrifolium, Euproteaciphyllum tridacnoides, Banksieaeformis sp., Banksieaephyllum sp., Banksieaephyllum sp., Banksieaephyllum orientalis sp., Banksieaephyllum linearis sp., Lomatia sp., Lomatia fraxinifolia, Lomatia xeromorpha sp.'),y('Angiospermae','Proteales','unknown','Proteacidites spp., Proteacidites reticulatus, cf. Proteacidites pseudomoides'),y('Angiospermae','Escalloniales','Escalloniaceae','Quintinia psilatispora'),y('Angiospermae','Santalales','Loranthaceae','Gothanipollis bassensis'),y('Angiospermae','Gunnerales','Gunneraceae','Tricolpites spp., Tricolpites reticulatus'),y('Angiospermae','Laurales','Lauraceae','Laurophyllum sp.'),y('Angiospermae','Liliales','unknown','Liliacidites sp.'),y('Angiospermae','Poales','Restionaceae','Milfordia hypolaenoides'),y('Angiospermae','Malvales','Sterculiaceae','aff. Brachychiton sp.'),y('Angiospermae','Fagales','Casuarinaceae','aff. Gymnostoma sp.'),y('Angiospermae','Fagales','unknown','Nothofagidites deminutus, Nothofagidites spp., Nothofagidites goniatus, Nothofagidites flemingii, Nothofagidites cf. brachyspinulosus, Nothofagidites falcatus, Nothofagidites asperus'),y('Angiospermae','Fagales','Nothofagaceae','Nothofagus aff. alessandrii, Nothofagus gunnii, Nothofagus cethanica sp., Nothofagus tasmanica'),y('Angiospermae','Oxalidales','Elaeocarpaceae','aff. Elaeocarpus sp.'),y('Angiospermae','Oxalidales','Cunoniaceae','Schizomeria tasmaniensis sp., Vesselowskya aff. ruinfolia, Callicoma serratifolia, Weinmanniaphyllum bernardii sp., Acsmithia grandiflora'),y('Angiospermae','Bennettitales','unknown','Ptilophyllum muelleri'),y('Pinopsida','Pinales','Araucariaceae','Agathis sp., Agathis sp., Agathis sp., Agathis sp., Agathis spp., Agathis brevigongylodes, Agathis vittatus, Araucaria sp., Araucariacites australis'),y('Magnoliopsida','unknown','unknown','Malvacipollis duratus, Malvacipollis subtilis, Periporopollenites polyoratus, Periporopollenites vesicus, Periporopollenites demarcatus, Haloragacidites harrisii, Tricolporites spp., Tricolporites cf. angurium'),y('Hymenophyllum','unknown','unknown','aff. Hymenophyllum sp.')],[y('Pteridopsida','Filicales','unknown','Cladophlebis vahliana, Cladophlebis usheri, Cladophlebis sp. A'),y('Dewalquea','unknown','unknown','Dewalquea sp.'),y('Coniferales','Coniferales','Cupressaceae','Thuites corpulentus, Glyptostrobus comoxensis, Metasequoia cuneata'),y('Cycadopsida','Cycadales','Nilssoniaceae','Pseudoctenis latipennis'),y('Cycadopsida','Cycadales','unknown','Nilsonia'),y('Asplenites','unknown','unknown','Asplenites tenellus'),y('Angiospermae','Typhales','Commelinidae','Sparganium sp.'),y('Dicotyledoneae','Ericales','Sapotaceae','Sapotacites sp.'),y('Dicotyledoneae','Myrtales','Combretaceae','Combretum cf. leve'),y('Angiospermae','Ranunculales','Menispermaceae','Menispermum dauricumoides, Menispermites torosus'),y('Angiospermae','Proteales','Platanaceae','Platanus affinis'),y('Angiospermae','Dipsacales','Caprifoliaceae','Viburnum insigne'),y('Angiospermae','Asterales','Asteraceae','Calycites'),y('Angiospermae','Arecales','Arecaceae','Geonomites imperialis'),y('Angiospermae','Sapindales','Sapindaceae','Cupanites crenularis, Sapindus pacificus'),y('Angiospermae','Rosales','Rhamnaceae','Rhamnites eminens, Zizyphus cretaceus, Zizyphus neillii'),y('Angiospermae','Rosales','Rosaceae','Chrysobalanus nervillosus'),y('Angiospermae','Rosids','Moraceae','Artocarpus occidentalis'),y('Angiospermae','Magnoliales','Magnoliaceae','Liriodendron succedens'),y('Pinopsida','Pinales','Araucariaceae','Dammarites robinsi'),y('Pinopsida','Pinales','Taxaceae','Amentotaxus cf. campbelli, Protophyllocladus polymorphia'),y('Magnoliopsida','unknown','unknown','Dicotylophyllum sp.'),y('Dombeyopsis','unknown','unknown','Dombeyopsis ovata')],[y('Dicotyledoneae','unknown','unknown','Araliopsoides cretacea'),y('Angiospermae','Proteales','Platanaceae','Platanus cissoides'),y('Angiospermae','Dipsacales','Caprifoliaceae','Viburnites Evansanus'),y('Angiospermae','Laurales','Lauraceae','Sassafras Mudgii'),y('Angiospermae','Fagales','Fagaceae','Quercus Wardiana'),y('Magnoliopsida','unknown','unknown','Celastrophyllum acutidens')],[y('Pteridopsida','unknown','unknown','Baculatisporites comaumensis, Laevigatosporites spp.'),y('Polypodiopsida','Polypodiales','Polypodiidae','Verrucatosporites spp.'),y('Polypodiopsida','Gleicheniales','Polypodiidae','Gleicheniidites circinidites'),y('Verrucosisporites','unknown','unknown','Verrucosisporites kopukuensis'),y('Coniferales','Coniferales','Podocarpaceae','Dacrycarpites australiensis, Podocarpidites spp., Microcachryidites antarcticus, Phyllocladidites mawsonii'),y('Angiospermae','Alismatales','Juncaginaceae','Triglochin spp.'),y('Dicotyledoneae','Ericales','Ericaceae','Ericipites spp.'),y('Dicotyledoneae','Myrtales','unknown','Myrtaceidites spp., Myrtaceidites mesonesus, Myrtaceidites parvus'),y('Angiospermae','Proteales','Proteaceae','Beaupreaidites arcuatus'),y('Angiospermae','Proteales','unknown','Proteacidites subscrabatus, Proteacidites ivanhoensis'),y('Angiospermae','Paracryphiales','Paracryphiaceae','Sphenostemon spp.'),y('Angiospermae','Santalales','Loranthaceae','Loranthaceae spp.'),y('Angiospermae','Chloranthales','Chloranthaceae','Ascarina spp.'),y('Angiospermae','Sapindales','Sapindaceae','Cupanieidites major, Cupanieidites orthoteichus'),y('Angiospermae','Fagales','unknown','Nothofagidites menziesii, Nothofagidites spp.'),y('Angiospermae','Fagales','Nothofagaceae','Nothofagus brassii, Nothofagus fusca'),y('Angiospermae','Malpighiales','Picrodendraceae','Micrantheum spinyspora'),y('Pinopsida','Pinales','Taxodiaceae','Cupressacites spp.'),y('Pinopsida','Pinales','Araucariaceae','Araucariacites australis'),y('Magnoliopsida','unknown','unknown','Dicotetradites spp., Malvacipollis spp., Periporopollenites spp., Haloragacidites harrisii')],[y('Pteridopsida','Filicales','unknown','Cladophlebis alberta, Cladophlebis cf. porsildi, Cladophlebis cf. virginiensis, Cladophlebis parva'),y('Gnetopsida','unknown','unknown','Pityophyllum nordenskioeldi'),y('Hepaticae','unknown','unknown','Thallites blairmorensis'),y('Coniferales','Coniferales','Podocarpaceae','Elatocladus acifolia, Podozamites sp.'),y('Coniferales','unknown','unknown','Athrotaxites sp.'),y('Ginkgoopsida','Caytoniales','Caytoniaceae','Sagenopteris cf. mclearni, Sagenopteris williamsii'),y('Ginkgoopsida','Ginkgoales','Ginkgoaceae','Ginkgo aff. digitata, Ginkgo cf. pluripartita'),y('Ctenopteris','unknown','unknown','Ctenopteris sp.'),y('Cycadopsida','Cycadales','Nilssoniaceae','Pseudoctenis aff. hazeltonensis'),y('Cycadopsida','Cycadales','unknown','Nilsonia'),y('Sphenopteridae','Sphenopteridae','unknown','Sphenopteris sp., Sphenopteris cf. latiloba'),y('Angiospermae','Apiales','Araliaceae','Aralia wellingtoniana'),y('Angiospermae','Laurales','Lauraceae','Sassafras parvofilium'),y('Angiospermae','Sapindales','Sapindaceae','Sapindopsis cf. magnifolia'),y('Angiospermae','Malpighiales','Salicaceae','Populites cf. dawsoni'),y('Angiospermae','Fabales','Fabaceae','Fontainea cf. grandiflora'),y('Angiospermae','Rosids','Moraceae','Ficus ovatifolia'),y('Angiospermae','Bennettitales','unknown','Ptilophyllum cf. columbianum, Ptilophyllum aff. montanense, Pterophyllum aff. plicatum, Pterophyllum cf. rectangulare, Pterophyllum sp., Pseudocycas cf. sp., Pseudocycas cf. unjiga'),y('Pinopsida','Pinales','Taxaceae','Elatides cf. splendida, Elatides cf. curvifolia, Cyparissidium gracile, Protophyllocladus subintegrifolius')],[y('','','','')],[y('','','','')]]
    assert len(Sediments_map) == len(Plants)
    Run_occurance(Rocks,Plants,Age_ranges,Sediments_map,Orders)
    return Plants

def rawfossils(clades):
    """
    Raw fossil information container
    """
    Spnames = ['Dichastopollenites sp.','Cretacaeiporites densimurus','Rosannia manika','Pseudowinterapollis couperi','Endressinia brasiliana','Virginianthus calycanthoides','Clavatipollenites sp.','Eichhornia sp.','Spirematospermum chandlerae','Eragrosites changii','Spinizonocolpites sp.','Phormium sp.','Liliacidites sp.','Nuhliantha nyanzaiana','Dioscorea sp.','Ceratophyllum sp.','Teixeiraea lusitanica','Platanocarpus brookensis','Tetracentronites panochetris','Hexaporotricolpites lamellaferus','Tricolpites sp.','Myricipites sp.','Cucurbitospermum obliquum','Eophylica priscastellata','Fontainea grandiflora','Celastrinites wardii','aff. Elaeocarpus sp.','Salix flexuosa','Guaiacum takliensis','Sterculia towneri','Akania sp.','Tapiscia serrata','Sapindopsis schultzii','Staphylea usheri','Myrtaceidites eugeniioides','Simarubinium mahurzari','Cissites ingens','Trochodendroides potomacensis','Dillenites paucidentatus','Syncolporites marginatus','Droseridites senonicus','Cornophyllum wardii','Natsiatum eocenicum','Eucommia serrata','Cordioxylon prototrichotoma','Cricotriporites sp.','Bignonites chalianus','Perfotricolpites digitatus','Ilex serrata','Quintinia psilatispora','Calycithes sp.','Viburnites Evansanus','Sphenostemon spp.','Aralia wellingtoniana','Caldesia brandoniana','Actinocalyx bohrii']
    Families = ['Nymphaeaceae','Trimeniaceae','Aristolochiaceae','Winteraceae','unknown','Calycanthaceae','Chloranthaceae','Pontederiaceae','Zingiberaceae','Poaceae','Arecaceae','Asparagaceae','unknown','Triuridaceae','Dioscoreaceae','Ceratophyllaceae','unknown','Platanaceae','unknown','Didymelaceae','Gunneraceae','Myricaceae','Cucurbitaceae','Rhamnaceae','Fabaceae','Celastraceae','Elaeocarpaceae','Salicaceae','Zygophyllaceae','Sterculiaceae','Akaniaceae','Tapisciaceae','Sapindaceae','Staphyleaceae','unknown','unknown','Vitaceae','Cercidiphyllacexae','Dilleniaceae','Loranthaceae','Droseraceae','Cornaceae','Icacinaceae','Eucommiaceae','Boraginaceae','Apocynaceae','Bignoniaceae','Convolvulaceae','Aquifoliaceae','Escalloniaceae','Asteraceae','Caprifoliaceae','Paracryphiaceae','Araliaceae','Alismataceae','Diapensiaceae']
    Body_plans = ['spore-pollen','spore-pollen','spore-pollen','spore-pollen','macrofossils','microfossils','spore-pollen','macrofossils','spore-pollen','macrofossils','microfossils','macrofossils','spore-pollen','spore-pollen','macrofossils','macrofossils','mesofossils','spore-pollen','macrofossils','spore-pollen','spore-pollen','spore-pollen','mesofossils','macrofossils (amber)','macrofossils','macrofossils','macrofossils','macrofossils','mesofossils','macrofossils','microfossils (adpression)','macrofossils','mesofossils','macrofossils','spore-pollen','macrofossils','macrofossils','macrofossils','macrofossils','spore-pollen','spore-pollen','mesofossils','mesofossils','macrofossils','macrofossils','spore-pollen','macrofossils','spore-pollen','macrofossils','microfossils','macrofossils','macrofossils','microfossils','macrofossils','macrofossils','macrofossils']
    Taxon_Opinions_References = ['https://doi.org/10.1080/01916122.2013.828662','https://doi.org/10.1080/01916122.2013.828662','https://doi.org/10.1371/journal.pone.0052455','https://www.ameghiniana.org.ar/index.php/ameghiniana/article/view/2521','https://doi.org/10.1086/423879','DOI: 10.1086/297217','https://doi.org/10.1080/01916122.2013.828662','R. Srivastava. 1991. A Catalogue of Fossil Plants from India Part 4: Cenozoic (Tertiary) Megafossils','https://doi.org/10.3133/b1300','https://doi.org/10.1007/BF02898918','https://doi.org/10.1016/j.gsf.2022.101457','https://doi.org/10.1016/j.coal.2009.07.009','S. Archangelsky. 1973. Palinología del Paleoceno de Chubut. I. Descripciones Sistemáticas. Ameghiniana 10(4):339-399','https://doi.org/10.3732/ajb.89.12.1940','http://hdl.handle.net/2346/65328','C. Linnaeus. 1753. Species Plantarum 1-1200','https://doi.org/10.1007/s00606-005-0347-z','https://doi.org/10.2307/2419407','https://doi.org/10.2307/2440448','DOI:10.2973/DSDP.PROC.38394041S.404.1978','D. M. Jarzen. 1982. Palynology of Dinosaur Provincial Park (Campanian) Alberta. Syllogeus 38:1-69','R. N. Lakhanpal, H. K. Maheshwari, and N. Awasthi. 1976. A Catalogue of Indian Fossil Plants. 1-318','https://doi.org/10.5962/bhl.title.110079','https://doi.org/10.1038/s41477-021-01091-w','https://doi.org/10.3133/b1300','https://doi.org/10.3133/b696','https://doi.org/10.1111/j.1469-8137.1976.tb01534.x','http://hdl.handle.net/2346/65328','C. Linnaeus. 1753. Species Plantarum 1-1200','https://doi.org/10.54991/jop.2008.238','https://doi.org/10.1086/430055','M. E. Collinson, S. R. Manchester, and V. Wilde. 2012. Fossil Fruits and Seeds of the Middle Eocene Messel biota, Germany. Abhandlungen der Senckenbergischen Gesellschaft Naturforschenden 570:1-249','http://hdl.handle.net/2346/65328','https://doi.org/10.3133/b696','S. C. Palma-Heldt, F. J. Fernandoy, I. Quezada and M. A. Leppe. 2004. Registro palinológico de Cabo Shirreff, Isla Livingstone, nueva localidad para el Mesozoico de las Shetland del Sur. V Simposio Argentino y I Latinoamericano sobre Investigaciones Antárticas','R. N. Lakhanpal, H. K. Maheshwari, and N. Awasthi. 1976. A Catalogue of Indian Fossil Plants. 1-318','http://hdl.handle.net/2346/65328','Y.-S. Liu, S.-X. Guo, and D. K. Ferguson. 1996. A catalogue of Cenozoic megafossil plants in China. Palaeontographica Abteilung B 238:141-179','http://hdl.handle.net/2346/65328','https://doi.org/10.1080/01916122.2011.633634','https://doi.org/10.1080/01916122.2013.828662','http://hdl.handle.net/2346/65328','DOI: 10.1126/science.146.3643.510','K. Ozaki. 1974. Miocene floras of the Pacific side of central Japan (I) Inkyoyama flora. Science Reports of the Yokohama National University, Section II 21:1-21','https://www.ameghiniana.org.ar/index.php/ameghiniana/article/view/747','DOI: 10.1127/palb/258/2001/87','https://revista.geologica.org.ar/raga/article/view/765','10.1016/0012-8252(68)90140-2','Y.-S. Liu, S.-X. Guo, and D. K. Ferguson. 1996. A catalogue of Cenozoic megafossil plants in China. Palaeontographica Abteilung B 238:141-179','https://doi.org/10.1111/j.1095-8339.1997.tb01404.x','https://nla.gov.au/nla.cat-vn4041561','http://hdl.handle.net/2346/65328','https://doi.org/10.1111/j.1469-8137.1976.tb01534.x','http://hdl.handle.net/2346/65328','https://doi.org/10.2307/2446086','https://doi.org/10.1016/0034-6667(85)90001-6']
    Age_References = ['https://doi.org/10.1080/01916122.2011.633634','https://doi.org/10.1016/S0899-5362(00)00026-9','https://doi.org/10.1371/journal.pone.0052455','DOI: 10.4072/rbp.2013.1.09','https://doi.org/10.1086/423879','DOI: 10.2307/2445980','https://doi.org/10.1080/01916122.1992.9989411','https://www.si.edu/object/siris_sil_458553','https://agris.fao.org/agris-search/search.do?recordID=US201300630567','https://doi.org/10.1007/BF02898918','http://www.scielo.org.ar/scielo.php?script=sci_arttext&pid=S1853-04002012000100003','https://doi.org/10.1016/j.coal.2009.07.009','https://doi.org/10.1111/1475-4983.00226','https://doi.org/10.2307/2657028','https://nla.gov.au/nla.cat-vn4041783','https://doi.org/10.1130/SPE287-p129','https://doi.org/10.1086/605120','https://doi.org/10.2307/2419407','V. M. Page. 1968. Angiosperm Wood from the Upper Creataceous of Central California: Part II. American Journal of Botany 55(2):168-172','Herngreen 1973','https://doi.org/10.1016/j.cretres.2005.11.012','https://www.researchgate.net/publication/287115477','https://doi.org/10.5962/bhl.title.110079','https://doi.org/10.1038/s41477-021-01091-w','G.B. Mellon, J.H. Hall, and C.R. Stelck. 1963. Lower Cretaceous Section, Belcourt Ridge, Northeastern British Columbia. Bulletin of Canadian Petroleum Geology 11:64-72','http://hdl.handle.net/2346/65328','https://doi.org/10.1006/cres.2000.0221','https://doi.org/10.3133/pp84','S. Hislop. 1861. Remarks on the geology of Nágpur. Journal of the Bombay Branch of the Royal Asiatic Society 6:194-204','https://doi.org/10.3133/pp129I','https://doi.org/10.1130/G23889A.1','A. Chandrasekharam. 1974. Megafossil flora from the Genesee Locality, Alberta, Canada. Palaeontographica Abteilung B 147:1-41','https://doi.org/10.2307/2399448','https://publications.gc.ca/site/eng/9.818646/publication.html','S. C. Palma-Heldt, F. J. Fernandoy, I. Quezada and M. A. Leppe. 2004. Registro palinológico de Cabo Shirreff, Isla Livingstone, nueva localidad para el Mesozoico de las Shetland del Sur. V Simposio Argentino y I Latinoamericano sobre Investigaciones Antárticas','R. N. Lakhanpal, H. K. Maheshwari, and N. Awasthi. 1976. A Catalogue of Indian Fossil Plants. 1-318','L.F. Ward. 1899. The Cretaceous formation of the Black Hills as indicated by the fossil plants. Cretaceous Formation of the Black Hills','W. A. Bell. 1956. Lower Cretaceous Floras of Western Canada. Geological Survey of Canada Memoir 285 (2528)1-153','https://doi.org/10.4095/101457','https://doi.org/10.1080/01916122.2011.633634','https://doi.org/10.1080/01916122.2011.633634','https://doi.org/10.5962/bhl.title.7646','DOI: 10.1126/science.134.3471.45','https://doi.org/10.1016/0031-0182(94)00109-L','https://www.ameghiniana.org.ar/index.php/ameghiniana/article/view/747','https://doi.org/10.1371/journal.pone.0052455','https://revista.geologica.org.ar/raga/article/view/765','10.1016/0012-8252(68)90140-2','K. R. Johnson. 2003. Denver Museum of Nature & Science archival records of Kirk Johnson fieldwork in Cretaceous rocks of the Rocky Mountain region','https://doi.org/10.1080/03115518408619610','https://publications.gc.ca/site/eng/9.818646/publication.html','L.F. Ward. 1899. The Cretaceous formation of the Black Hills as indicated by the fossil plants. Cretaceous Formation of the Black Hills','https://doi.org/10.1080/03115518708619139','G.B. Mellon, J.H. Hall, and C.R. Stelck. 1963. Lower Cretaceous Section, Belcourt Ridge, Northeastern British Columbia. Bulletin of Canadian Petroleum Geology 11:64-72','https://doi.org/10.2307/2446086','https://doi.org/10.1016/0034-6667(85)90001-6']
    Rocks = ['Well Khx-1, Muglad Basin, Sample Kh-17, Cretaceous to of Sudan','West Tiba-1, Zone PII, Sample 12, Cretaceous of Egypt','M3 - San Ramon section, Cretaceous of Argentina','Estancia San José, Cretaceous of Argentina','Crato Formation, Cretaceous of Brazil','Puddledock locality, Cretaceous to of the United States','Kokhav 2 Well, Core 3, Cretaceous to of Israel','Mohgaon Kalan, Paleogene of India','Neuse River Cut-Off, Cretaceous of the United States','Jianshangou bed, near Chaomidian, Cretaceous to of China','Bajo Comisión - Rio Mayer Formation, Cretaceous of Argentina','F45/f0394, middle Gore Lignite Measure, Newvale Mine, Oligocene to of New Zealand','Dakhla Oasis Area, Tda-2 DUPLICATE, Cretaceous of Egypt','Old Crossman Clay Pit, Cretaceous of the United States','Florissant Flora, Eocene of the United States','Belvidere, Cretaceous of the United States','Vale de Agua 320, Cretaceous to of Portugal','Bank near Brooke, Cretaceous to of the United States','Del Puerto Canyon, Cretaceous of the United States','Borehole 1-QS-1-MA, Sample 38, Cretaceous to of Brazil','CU 40 - 1989, Cretaceous of Chile','Wotoushan Formation (Jiaguan Formation), the Chishui area, Guizhou Province, Cretaceous of China','Lower Bagshot - Lake near Poole, Eocene of the United Kingdom','Burmese amber, Hkamti amber (QUST collection), Cretaceous of Myanmar','Upper Belcourt Ridge, Cretaceous of Canada','Extension 3771, Cretaceous to of Canada','Zamek Hill assemblage, Cretaceous to of Antarctica','Broken Arrow Bend, Cretaceous of the United States','Takli, Nagpur, Cretaceous of India','773 Hill Gould & Shattuck, Cretaceous of the United States','Palacio de los Loros 2, Paleocene of Argentina','Genesee, Paleocene of Canada','Aspen (2), Cretaceous of the United States','Protection Formation (Nanaimo Group), Cretaceous of Canada','Cabo Shirreff, Isla Livingston (Cretácico Inferior), Cretaceous of Antarctica','Mahurzari, Paleogene of India','Minnekahta Falls, Cretaceous of the United States','Upper Blairmore, Cretaceous of Canada','Comox Coalfield 3769, Cretaceous to of Canada','Well Azx-1, Muglad Basin, Sample Az-04, Cretaceous to of Sudan','Well Khx-1, Muglad Basin, Sample Kh-15, Cretaceous to of Sudan','P3651, Cretaceous of the United States','Reading Beds - Broxbourne, Paleocene of the United Kingdom','L.J. Hickey locality Bugger to get to (Cedar Mtn.), Paleocene of the United States','Puerto Visser, Paleocene of Argentina','D1 - San Ramon section, Paleocene of Argentina','Cerro Cachetamán, Cretaceous of Argentina','Mirador Formation, Norte de Santander, Eocene of Colombia','KJ9403 (DMNH 947), Cretaceous of the United States','Cethana Dam, Oligocene of Australia','Comox Coalfield 3768, Cretaceous to of Canada','Hot Springs 2, Cretaceous of the United States','Alcoa Anglesea locality, Site II , Lens A, Eocene of Australia','Upper Belcourt Ridge, Cretaceous of Canada','Forestdale, ca. 7 km east of Brandon, Brandon Lignite Formation, Vermont, Cenozoic of USA','Höganäs AB kaolin quarry at the Asen locality in the Kristianstad Basin, Scania, Cretaceous of Sweden']
    Distributions = ['Brazil, Egypt, South Sudan, Sudan, United States','Egypt','Argentina','Argentina','Brazil','United States','167 occurrences','Germany, India','71 occurrences','China','90 occurrences','New Zealand','269 occurrences','United States','7 occurrences','72 occurrences','Portugal','United States','United States','Brazil','576 occurrences','Antarctica, New Zealand, India, Canada, China','United Kingdom','Myanmar','Canada','Canada','Australia, Australia, Japan, Antarctica','United States','Bahamas, India','35 occurrences','Argentina','China, Germany, United States, Canada','Canada, Lebanon, United States','Canada, Lebanon, United States','Australia, Antarctica','India','United States','68 occurrences','United States, Canada','Colombia, Sudan, South Sudan','Egypt, Nigeria, Senegal, Sudan','Canada, United States','United Kingdom','80 occurrences','Argentina','Argentina, Colombia, Myanmar, Peru','Argentina','Venezuela, Peru, Myanmar, Colombia, Venezuela','167 occurrences','Australia','United States, Canada','United States','Australia','Canada','Unknown','Unknown']
    Numsp = len(Spnames)
    assert len(Families) == Numsp
    assert len(Orders) == Numsp
    assert len(Age_ranges) == Numsp
    assert len(Body_plans) == Numsp
    assert len(Taxon_Opinions_References) == Numsp
    assert len(Age_References) == Numsp
    assert len(Rocks) == Numsp
    assert len(Distributions) == Numsp
    assert len(Orders_Bounds) == Numsp
    Run_fossils(clades,Spnames,Families,Orders,Age_ranges,Body_plans,Taxon_Opinions_References,Age_References,Rocks,Distributions,Orders_Bounds)

def Run_fossils(clades,Spnames,Families,Orders,Age_ranges,Body_plans,Taxon_Opinions_References,Age_References,Rocks,Distributions,Orders_Bounds):
    """
    Conduct searching etc given input clades
    """
    Info_overall = {}
    for Ss,Fs,Os,As,Bs,Ts,Ags,Rs,Ds in zip(Spnames,Families,Orders,Age_ranges,Body_plans,Taxon_Opinions_References,Age_References,Rocks,Distributions):
        Info_overall[Os] = {'Species Name':Ss,'Familiy':Fs,'Age Range':As,'Body Plan':Bs,'Taxon Opinions&Reference':Ts,'Age Reference':Ags,'Rock':Rs,'Occurances':Ds,'Minimum Stem Bound':Orders_Bounds[Os]}
    info_givenclades = Info_overall.get(clades,{})
    info_clades(info_givenclades)

def Run_occurance(Rocks,Plants,Ages,Sediments_map,Orders):
    """
    Conduct cooccurance searching etc given input clades
    """
    cooccurance_info = Plants[Rocks]
    age,Sed,clade = Ages[Rocks],Sediments_map[Rocks],Orders[Rocks]
    cooccur_info(Rocks,cooccurance_info,age,Sed,clade)

def cooccur_info(Rocks,cooccurance_info,age,Sed,clade):
    """
    Show cooccurance info
    """
    df = pd.concat([pd.DataFrame.from_dict([i]) for i in cooccurance_info],ignore_index=True)
    print(df)
    logging.info(("Oldest fossil found for {}".format(clade),Sed,age))

def info_clades(info_givenclades):
    """
    Show logging information of given clades
    """
    for head,info in info_givenclades.items():
        logging.info("{0} : {1}\n".format(head,info))

def postprocesstree(treef):
    with open(treef,"r") as f:
        content = f.read().strip().strip('\n').replace(':0.00000','').replace("]","'").replace("[","'").replace("''","'")
    with open(treef,"w") as f:
        f.write(content)

def skipnontreeline(content):
    if not content[0].startswith("("): return skipnontreeline(content[1:])
    else: return content

def writedic(fil):
    dic = {}
    with open(orderdata,'r') as f:
        for line in f.readlines():
            content = line.strip().split(' ')
            dic[content[0]] = content[1]
    return dic

def gettreewithfossil(tree,formatt=False,wholetree=False,Yang=False,orderdata=None,Updated_Tree=False):
    with open(tree,'r') as f: lines = f.read()
    handle = StringIO(skipnontreeline(lines))
    Tree = Phylo.read(handle,"newick")
    treef = os.path.abspath(tree) + ".addtime"
    treeftax = os.path.abspath(tree) + ".addtime_taxonomy"
    taxonomy_dict = {}
    Focus_sp = None
    if orderdata is not None: sp_order = writedic(orderdata)
    for clade in Tree.get_terminals():
        if clade.name.endswith("_ap1"):
            clade.name = clade.name.replace("_ap1","")
            Focus_sp = clade.name
        if clade.name.endswith("_ap2"): clade.name = clade.name.replace("_ap2","")
        success = False
        while success == False:
            try:
                info = get_taxonomy(clade.name)
                success = True
            except HTTPError as e:
                success = False
        if info['order'] == '': info['order'] = clade.name # This step is tricky, assuming the input is already order name
        taxonomy_dict[clade.name] = info
        logging.info("{} belongs to {}".format(clade.name,taxonomy_dict[clade.name]['order']))
    for i,clade in enumerate(Tree.get_nonterminals()):
        order_occur = [taxonomy_dict[tip.name]['order'] for tip in clade.get_terminals()]
        if len(set(order_occur)) <=1: continue
        Mininum_limit,Maximum_limit,order_occur = 0,0,[]
        if wholetree:
            total_clades = getotalcaldes(clade,taxonomy_dict,Updated_Tree=Updated_Tree)
        else:
            total_clades = clade.get_terminals()
        for tip in total_clades:
            age = Orders_Bounds.get(taxonomy_dict[tip.name]['order'],0)/100 if not wholetree else Orders_Bounds.get(tip.name,0)/100
            if age > Mininum_limit: Mininum_limit = age
            if Yang:
                upper_age = Clades_Upper_Bounds_FollowYang.get(taxonomy_dict[tip.name]['order'],0)/100 if not wholetree else Clades_Upper_Bounds_FollowYang.get(tip.name,0)/100
            else:
                upper_age = Clades_Upper_Bounds.get(taxonomy_dict[tip.name]['order'],0)/100 if not wholetree else Clades_Upper_Bounds.get(tip.name,0)/100
            if upper_age > Maximum_limit: Maximum_limit = upper_age
        clade.name = '[>{0:.4f}<{1:.4f}]'.format(Mininum_limit,Maximum_limit)
    Phylo.write(Tree,treef, "newick")
    postprocesstree(treef)
    for clade in Tree.get_terminals(): clade.name = taxonomy_dict[clade.name]['order']
    Phylo.write(Tree,treeftax, "newick")
    postprocesstree(treeftax)
    if formatt:
        formatmcmctree(len(Tree.get_terminals()),treef)
        formatmcmctree(len(Tree.get_terminals()),treeftax)
    if Focus_sp !=None:
        formatfocussp(treef,Focus_sp)
        formatfocussp(treeftax,Focus_sp)
    return treef

def formatfocussp(fname,Focus_sp):
    with open(fname,"r") as f: content = f.read()
    with open(fname,"w") as f: f.write(content.replace(Focus_sp+",",Focus_sp+"_ap1,").replace(Focus_sp+")",Focus_sp+"_ap2)"))

def formatmcmctree(spnum,fname):
    with open(fname,"r") as f: content = f.read()
    with open(fname,"w") as f:
        f.write("{} 1\n{}".format(spnum,content))

def getotalcaldes(clade,taxonomy_dict,Updated_Tree=False):
    handle = StringIO(Updated_Tree_in_Paper) if Updated_Tree else StringIO(APG_IV_Modified_Tree)
    Tree = Phylo.read(handle,"newick")
    orrurred_clades = [taxonomy_dict[tip.name]['order'] for tip in clade.get_terminals()]
    MRCA_Node = Tree.common_ancestor(*orrurred_clades)
    return MRCA_Node.get_terminals()

def findneighbor(belonged_order,number=8):
    # The number here is the number of orders
    handle = StringIO(APG_IV_Modified_Tree)
    Tree = Phylo.read(handle,"newick")
    All_Clades = [i.name for i in Tree.get_terminals()]
    needed_mrca = None
    for mrca in map(lambda x:Tree.common_ancestor(x,belonged_order),All_Clades):
        if len(mrca.get_terminals()) == number:
             needed_mrca = mrca
             break
    return needed_mrca

def replacespecies(content,Clade_names,inname,onlyone=False):
    original_sp_num = int(content.split('\n')[0].split(' ')[0])
    for i in Clade_names:
        if i.endswith("_ap1") or i.endswith("_ap2"): continue
        #added_num = len(Orders_replace_Species[i].split(',')) - 1
        #final_sp_num = original_sp_num + added_num
        if onlyone:
            content = content.replace(i,Orders_replace_One_Species[i])
            logging.info("Representative species of {0}: {1}".format(i,Orders_replace_One_Species[i]))
        else:
            content = content.replace(i,Orders_replace_Species[i])
            logging.info("Representative species of {0}: {1}".format(i,Orders_replace_Species[i].replace('()','')))
    handle = StringIO(content.split("\n")[1])
    Tree = Phylo.read(handle,"newick")
    spnum = len(Tree.get_terminals())
    with open("{}_sp{}_species_list_space".format(inname,spnum),"w") as f:
        focussp = ''
        for i in Tree.get_terminals():
            if i.name.endswith("_ap2") or i.name.endswith("_ap1"):
                focussp = i.name[:-4]
                continue
            f.write("{} ".format(i.name))
        f.write("{}".format(focussp))
    with open("{}_sp{}_species_list_lines".format(inname,spnum),"w") as f:
        focussp = ''
        for i in Tree.get_terminals():
            if i.name.endswith("_ap2") or i.name.endswith("_ap1"):
                focussp = i.name[:-4]
                continue
            f.write("{}\n".format(i.name))
        f.write("{}".format(focussp))
    content = content.replace(str(original_sp_num)+" ",str(spnum)+" ")
    return content, spnum

def getproperstartingtree(treef,number,outdir,onlyone=False):
    # Only works for order-level/or more recent WGDs
    Tree = Phylo.read(treef,"newick")
    Clade_names = [i.name for i in Tree.get_terminals()]
    with open(treef,"r") as f: content = f.read()
    content, spnum = replacespecies(content,Clade_names,os.path.basename(treef),onlyone=onlyone)
    with open(treef+'_{}sp'.format(spnum),"w") as f: f.write(content)

def otherpaperestimate(data):
    maximum_other_paper = {'eudicots':0}

def reshuffleMSA(fn_msa):
    msa = AlignIO.read(fn_msa,"fasta")
    # first select a random species
    shuffled_sp = msa[np.random.randint(0,len(msa))]
    re_shuffled_sp = reshuffleseq(shuffled_sp)

def reshuffleseq(shuffled_sp):
    # sequence replacement can be synon and non-synon, we should maybe estimate/assume the distribution of w
    Original_sequence = str(shuffled_sp.seq)
    #for i in range(0,len(Original_sequence),3):
    # 3rd nt of codon, probabilty sampled from a lognormal distribution (Overall probability should follow lognormal distribution, mean=0.5, std=0.5)
    # An idea of using Bayesian inference to get an MLE of the proability, requires a prior distribution of proability of 1st, 2nd, 3rd nt changes, which can be obtained by codon-alignment, np.mean(total number of 1st nt changed /total number of codon)


#def simulateMaximumbound(starttree,speciespool):
    # starttree with tip names in order
